float4x4 matWorld;

#include "Shaders/ShadowMapping.fx"

float4x4    matVP;
float2      BlockOffset;
float3      CamPos;
float       BlendStart;
float       BlendDist;
float       rcpBlendDist;
float       Time;
float2      WindDir;
float       WaveStrength;
float       WaveSpeed;
float       LightStrength;
float       AmbientLight;

//textures
int TexFilter; 
int TexAnisotropy;

texture Texture;
sampler STex = sampler_state
{
    Texture = <Texture>;
    AddressU = Wrap;
    AddressV = Wrap;
    MipFilter = LINEAR;
    MagFilter = LINEAR;
    MinFilter = (TexFilter);
    MaxAnisotropy = (TexAnisotropy);
};

struct VS_INPUT
{
    float3  Pos : POSITION;
    float2  Tex : TEXCOORD0;
    float   TimeOffset : TEXCOORD1;
    float   BlendPos : TEXCOORD2;
};

struct VS_OUTPUT
{
    float4 Pos : POSITION;
    float2 Tex : TEXCOORD0;
    float  Alpha : TEXCOORD1;
    float4 SMTexC: TEXCOORD2;
    float2 Depth : TEXCOORD3;
};

VS_OUTPUT VS(VS_INPUT i, uniform bool bShadow)
{
    VS_OUTPUT o;
    
    float4 p = float4(i.Pos,1);
    p.xy += BlockOffset;
    
    //do wave animation...
    float time = i.TimeOffset + Time;
    float s = cos(time*3.1415*2*WaveSpeed);
    p.xy += s*WindDir*WaveStrength * (1-i.Tex.y); //tv indicates the height of the vertice
    
    o.Pos = mul(p,matVP);
    
    o.Tex = i.Tex;

    if(bShadow)
    {
        o.SMTexC = CalcSMTexC(p);
        o.Depth = CalcDepth(p);
    }
    else
    {
        o.SMTexC = 0;
        o.Depth = 0;
    }

    if(length(p.xyz - CamPos) > BlendStart + i.BlendPos * BlendDist)
        o.Alpha = 0;
    else
        o.Alpha = 1;
    
    return o;
}

float4 PS(VS_OUTPUT i, uniform bool bShadow) : COLOR
{
    float shadow;
    if(bShadow)
        shadow = CalcShadow_Simple(i.SMTexC, i.Depth.x/i.Depth.y);
    else
        shadow = 0;
    float4 col = tex2D(STex,i.Tex);
    col.rgb *= (1-shadow) * LightStrength + AmbientLight;
    col.a *= i.Alpha;
    return col;
}

technique Render
{
    pass P0
    {
        VertexShader = compile vs_2_0 VS(false);
        PixelShader  = compile ps_2_0 PS(false);
    }
}

technique Render_Shadow
{
    pass P0
    {
        VertexShader = compile vs_2_0 VS(true);
        PixelShader  = compile ps_2_0 PS(true);
    }
}


