static const float3 LUM_VECTOR = {0.3f, 0.58f, 0.12f};

float2 g_InputSize;

float g_BrightPassOffset;
float g_BrightPassThresshold;
float g_Luminance;
float g_Key;

float g_Rho;
float g_GaussStrength;
float g_OffsetScale;

float g_E;
float g_Phi;

texture TexDownInput;
sampler2D sDownInput = sampler_state
{
	Texture = <TexDownInput>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = POINT;
    MinFilter = POINT;
};

texture TexScene;
sampler2D sScene = sampler_state
{
	Texture = <TexScene>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = POINT;
    MinFilter = POINT;
};

texture TexDown;
sampler2D sDown = sampler_state
{
	Texture = <TexDown>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = POINT;
    MinFilter = POINT;
};

texture TexLumInit;
sampler2D sLumInit = sampler_state
{
	Texture = <TexLumInit>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = POINT;
    MinFilter = POINT;
};

texture TexLumDown;
sampler2D sLumDown = sampler_state
{
	Texture = <TexLumDown>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = POINT;
    MinFilter = POINT;
};

texture TexFilterInput;
sampler2D sFilterInput = sampler_state
{
	Texture = <TexFilterInput>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = POINT;
    MinFilter = POINT;
};

texture g_texLocalScale0, g_texLocalScale1, g_texLocalScale2, g_texLocalScale3, g_texLocalScale4, g_texLocalScale5, g_texLocalScale6;
sampler g_sLocalScale[] = 
{
	sampler_state {	Texture = <g_texLocalScale0>; },
	sampler_state { Texture = <g_texLocalScale1>; },
	sampler_state { Texture = <g_texLocalScale2>; },
	sampler_state { Texture = <g_texLocalScale3>; },
	sampler_state { Texture = <g_texLocalScale4>; },
	sampler_state { Texture = <g_texLocalScale5>; },
	sampler_state { Texture = <g_texLocalScale6>; }
};

float4 PSDown4(float2 tc : TEXCOORD0) : COLOR
{
	float4 SumColor = 0;

	for(int y = 0; y < 4; y++)
	{
		for(int x = 0; x < 4; x++)
		{
			SumColor += tex2D(sDownInput, tc + float2(x - 1.5f, y - 1.5f) / g_InputSize);
		}
	}

	return SumColor / 16;
}

float4 PSDown2(float2 tc : TEXCOORD0) : COLOR
{
	float4 SumColor = 0;

	for(int y = 0; y < 2; y++)
	{
		for(int x = 0; x < 2; x++)
		{
			SumColor += tex2D(sDownInput, tc + float2(x - 0.5f, y - 0.5f) / g_InputSize);
		}
	}

	return SumColor / 4;
}

float4 LumInitPS(float2 tc : TEXCOORD0) : COLOR
{
	float LogLumSum = 0;

	for(int y = 0; y < 4; y++)
	{
		for(int x = 0; x < 4; x++)
		{
			// Compute the sum of log(luminance) throughout the sample points
			// a 8*64 sized input texture is assumed, because it wont be the exactly size (rather biger),
			// some pixel will be left out. however it wont change the final result much.
			//float2 stc = tc + float2(x - 3.5f, y - 3.5f) / (8 * 64);
			//LogLumSum += log(dot(tex2D(sScene, stc), LUM_VECTOR) + 0.00001f);

			float2 stc = tc + float2(x - 1.5f, y - 1.5f) / (4 * 64);
			LogLumSum += log(dot(tex2D(sDown, stc), LUM_VECTOR) + 0.00001f);
		}
	}

	return LogLumSum / 64.f;
}

float4 LumDownPS(float2 tc : TEXCOORD0) : COLOR
{
	float ResampleSum = 0.0f; 
    
	for(int y = 0; y < 8; y++)
	{
		for(int x = 0; x < 8; x++)
		{
			// Compute the sum of luminance throughout the sample points
			float2 stc = tc + float2(x-3.5f, y-3.5f) / 64.f;
			ResampleSum += tex2D(sLumInit, stc);
		}
	}
        
    // Divide the sum to complete the average
    return ResampleSum / 64;
}

float4 LumFinalPS(float2 tc : TEXCOORD0) : COLOR
{
	float ResampleSum = 0.0f; 
    
	for(int y = 0; y < 8; y++)
	{
		for(int x = 0; x < 8; x++)
		{
			// Compute the sum of luminance throughout the sample points
			float2 stc = tc + float2(x-3.5f, y-3.5f) / 8.f;
			ResampleSum += tex2D(sLumDown, stc);
		}
	}
        
    // Divide the sum to complete the average, and perform an exp() to complete
    // the average luminance calculation
	return exp(ResampleSum / 64);
}

float4 BrightPassPS(float2 tc : TEXCOORD0) : COLOR
{
	float4 vSample = tex2D(sDown, tc);
	
	// Determine what the pixel's value will be after tone-mapping occurs
	vSample.rgb *= g_Key / (g_Luminance + 0.001f);
	
	// Subtract out dark pixels
	vSample.rgb -= g_BrightPassThresshold;
	
	// Clamp to 0
	vSample = max(vSample, 0.0f);
	
	// Map the resulting value into the 0 to 1 range. Higher values for
	// BRIGHT_PASS_OFFSET will isolate lights from illuminated scene 
	// objects.
	vSample.rgb /= (g_BrightPassOffset + vSample);
    
	return vSample;
	//return float4(1,0,vSample.r,1);
}

float4 GaussPS(float2 tc : TEXCOORD0, uniform int SIZE) : COLOR
{
	float4 ColorSum = float4(0,0,0,0);
	float SumWheight = 0;
	
	for(int y = -SIZE; y <= SIZE; y++)
	{
		for(int x = -SIZE; x <= SIZE; x++)
		{
			[branch]if (x*x + y*y <= SIZE*SIZE)
			{                  
				float2 stc = tc + float2(x, y) / g_InputSize;
                float gauss = 1 / (2 * 3.141592654f * g_Rho * g_Rho) * exp(-0.5 * (x * x * g_OffsetScale * g_OffsetScale + y * y * g_OffsetScale * g_OffsetScale) / (g_Rho * g_Rho)); 
				ColorSum += tex2Dlod(sFilterInput, float4(stc,0,0)) * gauss;
				SumWheight += gauss;
			}
        }
    }
	
	//normalize and scale to strength
	return ColorSum * g_GaussStrength / SumWheight;
}

float4 LumScalePS(float2 tc : TEXCOORD0) : COLOR
{
    float4 vSample = tex2D(sScene, tc);

    //Linear Mapping
    vSample.rgb *= g_Key / (g_Luminance + 0.0001f);
    
	return vSample;
}

float4 BuildLocalAdaptMapPS(float2 tc : TEXCOORD0) : COLOR
{
	float4 vi = 0;
    float4 vilast = tex2D(g_sLocalScale[0], tc);
        	
	for (int i = 1; i < 7; i++)
	{
		int scale = pow(2, (int)(i / 2));
		vi = tex2D(g_sLocalScale[i], tc);
		
		float s = 1 / (2 * sqrt(2)) * pow(1.6, i);
		
		float vs = dot(abs(vilast - vi), LUM_VECTOR) / (pow(2, g_Phi) * g_Key / (s * s));
			
		if(vs > g_E)
			break;
		
		vilast = vi; 
	}
		 
	return vilast;	
}

stateblock RenderStates = stateblock_state
{
	ZEnable = false;
    Cullmode = None;
    AlphaBlendEnable = false;
    AlphaTestEnable = false; 
    
};

technique Down4
{
	pass P0
	{
		StateBlock = (RenderStates);

		VertexShader = null;
        PixelShader  = compile ps_3_0 PSDown4();
	}
}

technique Down2
{
	pass P0
	{
		StateBlock = (RenderStates);

		VertexShader = null;
        PixelShader  = compile ps_3_0 PSDown2();
	}
}

technique LumInit
{
	pass P0
	{
		StateBlock = (RenderStates);

		VertexShader = null;
        PixelShader  = compile ps_3_0 LumInitPS();
	}
}

technique LumDown
{
	pass P0
	{
		StateBlock = (RenderStates);

		VertexShader = null;
        PixelShader  = compile ps_3_0 LumDownPS();
	}
}

technique LumFinal
{
	pass P0
	{
		StateBlock = (RenderStates);

		VertexShader = null;
        PixelShader  = compile ps_3_0 LumFinalPS();
	}
}

technique BrightPass
{
	pass P0
	{
		StateBlock = (RenderStates);

		VertexShader = null;
        PixelShader  = compile ps_3_0 BrightPassPS();
	}
}

technique Gauss5x5
{
	pass P0
	{
		StateBlock = (RenderStates);

		VertexShader = null;
        PixelShader  = compile ps_3_0 GaussPS(2);
	}
}

technique Gauss9x9
{
	pass P0
	{
		StateBlock = (RenderStates);

		VertexShader = null;
        PixelShader  = compile ps_3_0 GaussPS(4);
	}
}

technique LumScale
{
	pass P0
	{
		StateBlock = (RenderStates);

		VertexShader = null;
        PixelShader  = compile ps_3_0 LumScalePS();
	}
}

technique BuildLocalAdaptMap
{
	pass P0
	{
		StateBlock = (RenderStates);

		VertexShader = null;
        PixelShader  = compile ps_3_0 BuildLocalAdaptMapPS();
	}
}


