#define MAXDIR 2
#define MAXDSPOT 5
#define MAXPOINT 5

#define LT_DIRECTIONAL 1
#define LT_POINT 2
#define LT_SPOT 3

#define BASEPARAM	\
	float4 Color;	\
	float bSM;		\
	float SMSize;	\
	float SlopeBias;\
	float ConstBias;\
	float Specular;	\
	float4x4 MatSM;	\

struct DirLightParam
{
	BASEPARAM
	
	float3 Direction;
};

struct SpotLightParam
{
	BASEPARAM

	float3 Position;
	float3 Direction;
	float3 SpotParam;
	float3 Attenuation;
};

struct PointLightParam
{
	BASEPARAM

	float3 Position;
	float3 Attenuation;
};

DirLightParam g_DirLights[MAXDIR];
SpotLightParam g_SpotLights[MAXDSPOT];
PointLightParam g_PointLights[MAXPOINT];

//shadow maps and samples
texture g_DirLightSM0, g_DirLightSM1;
texture g_SptLightSM0, g_SptLightSM1, g_SptLightSM2, g_SptLightSM3, g_SptLightSM4;
texture g_PntLightSM0, g_PntLightSM1, g_PntLightSM2, g_PntLightSM3, g_PntLightSM4;

sampler g_sDirLightSM[] = 
{
	sampler_state
	{
		Texture = <g_DirLightSM0>;
		AddressU = Clamp;
		AddressV = Clamp;
		MipFilter = NONE;
		MagFilter = POINT;
		MinFilter = POINT;
	},
	sampler_state
	{
		Texture = <g_DirLightSM1>;
		/*AddressU = Clamp;
		AddressV = Clamp;
		MipFilter = NONE;
		MagFilter = POINT;
		MinFilter = POINT;*/
	}
};

/*sampler g_sSpotLightSM[] = {
sampler_state{Texture = <g_SptLightSM0>;},
sampler_state{Texture = <g_SptLightSM1>;},
sampler_state{Texture = <g_SptLightSM2>;},
sampler_state{Texture = <g_SptLightSM3>;},
sampler_state{Texture = <g_SptLightSM4>;}};

sampler g_sPointLightSM[] = {
sampler_state{Texture = <g_PntLightSM0>;},
sampler_state{Texture = <g_PntLightSM1>;},
sampler_state{Texture = <g_PntLightSM2>;},
sampler_state{Texture = <g_PntLightSM3>;},
sampler_state{Texture = <g_PntLightSM4>;}};*/

//this will be preshadered
float4x4 GetMatSMTex(uniform float4x4 matSM, uniform float SMSize)
{
	float off = 0.5f + 0.5 / SMSize;
	//float off = 0.5f + 0.5f * SM_rcpShadowMapSize;
	return mul(matSM, float4x4(	0.5f, 0.0f, 0.0f, 0.0f,
									0.0f, -0.5f, 0.0f, 0.0f,
									0.0f, 0.0f, 1, 0.0f,
									off, off, 0.0f, 1.0f));
}

float2 CalcSMTexC(const float3 p, uniform float4x4 matSM, uniform float SMSize)
{
	const float4 tcproj = mul(float4(p, 1), GetMatSMTex(matSM, SMSize));
	return tcproj.xy / tcproj.w;
}

float CalcShadow_Simple(const float2 ShadowMapTexC, const float Depth, uniform sampler sShadowMap)
{
	return ((tex2D(sShadowMap, ShadowMapTexC)).r < Depth)? 1.0f: 0.0f;  
}

float CalcShadow_4xPCF(const float2 ShadowMapTexC, const float Depth, uniform sampler sShadowMap, uniform float SMSize)
{
	float2 texelpos = ShadowMapTexC * SMSize;
    float2 lerps = frac( texelpos );

	const float rcpShadowMapSize = 1 / SMSize;

	float sourcevals[4];
	sourcevals[0] = (tex2D( sShadowMap, ShadowMapTexC ).r < Depth)? 1.0f: 0.0f;  
    sourcevals[1] = (tex2D( sShadowMap, ShadowMapTexC + float2(rcpShadowMapSize, 0) ).r < Depth)? 1.0f: 0.0f;  
    sourcevals[2] = (tex2D( sShadowMap, ShadowMapTexC + float2(0, rcpShadowMapSize) ).r < Depth)? 1.0f: 0.0f;  
    sourcevals[3] = (tex2D( sShadowMap, ShadowMapTexC + float2(rcpShadowMapSize, rcpShadowMapSize) ).r < Depth)? 1.0f: 0.0f;  
        
    // lerp between the shadow values to calculate the light amount
    return lerp( lerp( sourcevals[0], sourcevals[1], lerps.x ), lerp( sourcevals[2], sourcevals[3], lerps.x ), lerps.y );
}

float CalcShadow_16xPCF(const float2 ShadowMapTexC, const float Depth, uniform sampler sShadowMap, uniform float SMSize)
{
	float2 texelpos = ShadowMapTexC * SMSize;
    float2 lerps = frac( texelpos );

	const float rcpShadowMapSize = 1 / SMSize;

	float sourcevals[4];
	sourcevals[0] = (tex2D( sShadowMap, ShadowMapTexC ).r < Depth)? 1.0f: 0.0f;  
    sourcevals[1] = (tex2D( sShadowMap, ShadowMapTexC + float2(rcpShadowMapSize, 0) ).r < Depth)? 1.0f: 0.0f;  
    sourcevals[2] = (tex2D( sShadowMap, ShadowMapTexC + float2(0, rcpShadowMapSize) ).r < Depth)? 1.0f: 0.0f;  
    sourcevals[3] = (tex2D( sShadowMap, ShadowMapTexC + float2(rcpShadowMapSize, rcpShadowMapSize) ).r < Depth)? 1.0f: 0.0f;  

	sourcevals[0] += (tex2D( sShadowMap, ShadowMapTexC + float2(-rcpShadowMapSize, -rcpShadowMapSize)).r < Depth)? 1.0f: 0.0f;  
	sourcevals[0] += (tex2D( sShadowMap, ShadowMapTexC + float2(0, -rcpShadowMapSize)).r < Depth)? 1.0f: 0.0f;  
	sourcevals[0] += (tex2D( sShadowMap, ShadowMapTexC + float2(-rcpShadowMapSize, 0)).r < Depth)? 1.0f: 0.0f;  

	sourcevals[1] += (tex2D( sShadowMap, ShadowMapTexC + float2(rcpShadowMapSize, -rcpShadowMapSize)).r < Depth)? 1.0f: 0.0f;  
	sourcevals[1] += (tex2D( sShadowMap, ShadowMapTexC + float2(2 * rcpShadowMapSize, -rcpShadowMapSize)).r < Depth)? 1.0f: 0.0f;  
	sourcevals[1] += (tex2D( sShadowMap, ShadowMapTexC + float2(2 * rcpShadowMapSize, 0)).r < Depth)? 1.0f: 0.0f;  

	sourcevals[2] += (tex2D( sShadowMap, ShadowMapTexC + float2(-rcpShadowMapSize, rcpShadowMapSize)).r < Depth)? 1.0f: 0.0f;  
	sourcevals[2] += (tex2D( sShadowMap, ShadowMapTexC + float2(-rcpShadowMapSize, 2 * rcpShadowMapSize)).r < Depth)? 1.0f: 0.0f;  
	sourcevals[2] += (tex2D( sShadowMap, ShadowMapTexC + float2(0, 2 * rcpShadowMapSize)).r < Depth)? 1.0f: 0.0f;  

	sourcevals[3] += (tex2D( sShadowMap, ShadowMapTexC + float2(2 * rcpShadowMapSize, rcpShadowMapSize)).r < Depth)? 1.0f: 0.0f;  
	sourcevals[3] += (tex2D( sShadowMap, ShadowMapTexC + float2(rcpShadowMapSize, 2 * rcpShadowMapSize)).r < Depth)? 1.0f: 0.0f;  
	sourcevals[3] += (tex2D( sShadowMap, ShadowMapTexC + float2(2 * rcpShadowMapSize, 2 * rcpShadowMapSize)).r < Depth)? 1.0f: 0.0f;  
        
	sourcevals[0] *= 0.25f;
	sourcevals[1] *= 0.25f;
	sourcevals[2] *= 0.25f;
	sourcevals[3] *= 0.25f;

	return (sourcevals[0] + sourcevals[1] + sourcevals[2] + sourcevals[3]) * 0.25f;
    // lerp between the shadow values to calculate the light amount
    //return lerp( lerp( sourcevals[0], sourcevals[1], lerps.x ), lerp( sourcevals[2], sourcevals[3], lerps.x ), lerps.y );
}

float CalcShadow_16xPoisson(const float2 ShadowMapTexC, const float Depth, uniform sampler sShadowMap, uniform float SMSize)
{
	const float rcpShadowMapSize = 1 / SMSize;

	static float tc[16];
	tc[0] = float2(0.007937789, 0.73124397);
	tc[1] = float2(-0.10177308, -0.6509396);
	tc[2] = float2(-0.9906806, -0.63400936);
	tc[3] = float2(-0.5583586, -0.3614012);
	tc[4] = float2(0.7163085, 0.22836149);
	tc[5] = float2(-0.65210974, 0.37117887);
	tc[6] = float2(-0.12714535, 0.112056136);
	tc[7] = float2(0.48898065, -0.66669613);
	tc[8] = float2(-0.9744036, 0.9155904);
	tc[9] = float2(0.9274436, -0.9896486);
	tc[10] = float2(0.9782181, 0.90990245);
	tc[11] = float2(0.96427417, -0.25506377);
	tc[12] = float2(-0.5021933, -0.9712455);
	tc[13] = float2(0.3091557, -0.17652994);
	tc[14] = float2(0.4665941, 0.96454906);
	tc[15] = float2(-0.461774, 0.9360856);

	float sumTest = 0;
	for(int i = 0; i < 16; i++)
	{
		sumTest += (tex2D( sShadowMap, ShadowMapTexC + tc[i] * float2(rcpShadowMapSize, rcpShadowMapSize)).r < Depth)? 1.0f: 0.0f;  
	}

	return sumTest / 16;
}

float CalcDepth(const float3 p, const float4x4 matSM)
{
	float2 DepthZW = (mul(float4(p, 1), matSM)).zw;
	return DepthZW.x / DepthZW.y;
}

float LightAttenuation(const float dist, const float3 att)
{
	return 1 / (att.x + att.y*dist + att.z*dist*dist);
}

float CalcDirLightShadow(const int l, const float DotLN, const float3 WorldPos, const float CamDist)
{
	if(g_DirLights[l].bSM != 0)
	{
		//calculate depth bias
		float Slope = saturate(1 - DotLN);
		float Bias = Slope * g_DirLights[l].SlopeBias + g_DirLights[l].ConstBias;
		//Bias *= (0.0001f + log10(CamDist / 2000 + 1) * 0.09999f) * 500;
		Bias *= (0.001 + log10(CamDist / 1000 + 1)) * 30 * 4096 / g_DirLights[l].SMSize;
		float3 WorldPosBiased = WorldPos - g_DirLights[l].Direction * Bias;

		//sample shadow depth map and do shadow test
		float2 SMTC = CalcSMTexC(WorldPosBiased, g_DirLights[l].MatSM, g_DirLights[l].SMSize);
		float Depth = CalcDepth(WorldPosBiased, g_DirLights[l].MatSM);

		return CalcShadow_16xPoisson(SMTC, Depth, g_sDirLightSM[l], g_DirLights[l].SMSize);
		//return CalcShadow_16xPCF(SMTC, Depth, g_sDirLightSM[l], g_DirLights[l].SMSize);
		//return CalcShadow_4xPCF(SMTC, Depth, g_sDirLightSM[l], g_DirLights[l].SMSize);
		//return CalcShadow_Simple(SMTC, Depth, g_sDirLightSM[l]);
	}
	else
	{
		return 0;
	}
}

void DoDirLight(const int l, const float3 WorldPos, const float3 Normal, const float3 ViewVec, const float CamDist, uniform float specularpower, inout float3 iDiffuse, inout float3 iSpecular)
{
	//calc lightvec in tangent space
	float3 LightVec = normalize(-g_DirLights[l].Direction);

	//get factor of lightstrength by dot(L, N)
	float DotLN = dot(Normal, LightVec);

	if(DotLN > 0)
	{
		float Shadow = CalcDirLightShadow(l, DotLN, WorldPos, CamDist);
		//float Shadow = 0;

		//calc reflection vector
		float3 ReflVec = normalize(2 * DotLN * Normal - LightVec);

		//calc lighting
		float3 Lighting = g_DirLights[l].Color.rgb * (1 - Shadow);

		//do diffuse and specular ligthing
		iDiffuse += DotLN * Lighting;
		iSpecular += pow(saturate(dot(ReflVec, ViewVec)), specularpower) * Lighting * g_DirLights[l].Specular;
	}
}

float CalcSpotLightShadow(const int l, const float DotLN, const float3 WorldPos)
{
	if(g_DirLights[l].bSM  != 0)
	{
		//calculate depth bias
		float Slope = 1 - DotLN;
		float Bias = Slope * g_SpotLights[l].SlopeBias + g_SpotLights[l].ConstBias;

		//sample shadow depth map and do shadow test
		float2 SMTC = CalcSMTexC(WorldPos, g_SpotLights[l].MatSM, g_SpotLights[l].SMSize);
		float Depth = CalcDepth(WorldPos, g_SpotLights[l].MatSM) + Bias;
		return 0;//CalcShadow_4xPCF(SMTC, Depth, g_sSpotLightSM[l], g_SpotLights[l].SMSize);
	}
	else
	{
		return 0;
	}
}

void DoSpotLight(const int l, const float3 WorldPos, const float3 Normal, const float3 ViewVec, uniform float specularpower, inout float3 iDiffuse, inout float3 iSpecular)
{
	//calc lightvec in tangent space
	float3 LightVec = normalize(g_SpotLights[l].Position - WorldPos);

	//get factor of lightstrength by dot(L, N)
	float DotLN = saturate(dot(Normal, LightVec));
	
	float Shadow = CalcSpotLightShadow(l, DotLN, WorldPos);
	
	//distance to light
	float LightDist = length(g_SpotLights[l].Position - WorldPos);	

	float Att = LightAttenuation(LightDist, g_SpotLights[l].Attenuation);

	//spot cone computation, calculate SpotLightDirection in tangent space
	float3 LightDir = -g_SpotLights[l].Direction;
	//spot open angle
	float rho = dot(LightVec, LightDir);
	//apply attenuation depending on open angle
	Att *= pow( saturate( (rho - g_SpotLights[l].SpotParam.y) / (g_SpotLights[l].SpotParam.x - g_SpotLights[l].SpotParam.y) ), g_SpotLights[l].SpotParam.z );

	//calc reflection vector
	float3 ReflVec = normalize(2 * DotLN * Normal - LightVec);

	//calc lighting
	float3 Lighting = g_SpotLights[l].Color.rgb * (1 - Shadow) * Att;

	//do diffuse and specular ligthing
	iDiffuse += DotLN * Lighting;
	iSpecular += pow(saturate(dot(ReflVec, ViewVec)), specularpower) * Lighting * g_SpotLights[l].Specular;
}

float CalcPointLightShadow(const int l, const float DotLN, const float LightDist)
{
	if(g_DirLights[l].bSM  != 0)
	{
		//apply a bias to the worldposition
		float Slope = 1 - DotLN;
		float Bias = Slope * g_PointLights[l].SlopeBias + g_PointLights[l].ConstBias;
		float Depth = LightDist + Bias;
		//do shadow cubemap lookup and make shadow test
		return 0;//((texCUBE(g_sPointLightSM[l], LightVec)).r < Depth)? 1.0f: 0.0f;  
	}
	else
	{
		return 0;
	}
}

void DoPointLight(const int l, const float3 WorldPos, const float3 Normal, const float3 ViewVec, uniform float specularpower, inout float3 iDiffuse, inout float3 iSpecular)
{
	//distance to light
	float LightDist = length(g_PointLights[l].Position - WorldPos);

	//calc lightvec in tangent space
	float3 LightVec = normalize(g_PointLights[l].Position - WorldPos);

	//get factor of lightstrength by dot(L, N)
	float DotLN = saturate(dot(Normal, LightVec));

	float Shadow = CalcPointLightShadow(l, DotLN, LightDist);
		
	//calculate light attenuation
	float Att = LightAttenuation(LightDist, g_PointLights[l].Attenuation);

	//calc reflection vector
	float3 ReflVec = normalize(2 * DotLN * Normal - LightVec);

	//calc lighting
	float3 Lighting = g_PointLights[l].Color.rgb * (1 - Shadow) * Att;

	//do diffuse and specular ligthing
	iDiffuse += DotLN * Lighting;
	iSpecular += pow(saturate(dot(ReflVec, ViewVec)), specularpower) * Lighting * g_PointLights[l].Specular;
}