//define FRAMHIERARCHY is the per-frame matrix should be used
//define LIGHTING to enable all kinds off ligthing, otherwise the material is ambient only
//set the define NUMTEXCOORDSETS to the number of available and used input texture coordinate sets
//define COLORMAP to enable the material color map
//define SPECULARMAP to enable the material specular map
//define EMISSIVEMAP to enable the mateiral emissive map
//define REFLECTIONMAP
//define REFRACTIONMAP
//define NORMALMAP
//define HEIGHTMAP

//#define SCATTERING
//#define DEPTHTOALPHA
//#define DEFERRED
//#define ALPHAENABLE define when alphablending or alphatesting should be used
//#define RAYTRACEDREF

#ifndef MAPPINGTYPE
#define MAPPINGTYPE 0
#endif

//material texture mapping type
#define MT_NONE 0
#define MT_BUMP 1
#define MT_NORMAL 2
#define MT_PARALLAX 3
#define MT_RELIEF 4

#ifndef NUMTEXCOORDSETS
#define NUMTEXCOORDSETS 1
#endif

//material map lookup type
#define MTLT_NONE 0
#define MTLT_TEXCOORD 1
#define MTLT_SCREENCOORD 2
#define MTLT_SURFACENORMAL 3

struct STMaterial
{
	float3 AmbientColor;
	float3 DiffuseColor; float Opacity;
	float3 SpecularColor; float SpecularPower;
	float3 EmissiveColor; float EmissiveStrength;
}g_Material;

struct STBlending
{
	int AlphaBlendEnable;
	int AlphaTestEnable;
	int AlphaRef;
}g_Blending;

float4 g_AmbientLight : AMBIENT;
float3 g_CamPos : CAMPOS;

float4x4 g_matVP : VIEWPROJ;
float4x4 g_matInvVP : INVVIEWPROJ;
float4x4 g_matView : VIEW;
float4x4 g_matProj : PROJ;
float4x4 g_matWorld;
float4x4 g_matFrame;

int g_TexFilter : TEXFILTER; 
int g_TexAnisotropy : TEXANISO;
int g_CullMode : CULLMODE; // = 3; //CCW

//#include "Shaders\ShadowMapping.fx"
#ifdef SCATTERING
#include "Shaders\Scattering.fx"
#endif

#ifdef LIGHTING
#ifndef DEFERRED
#include "Shaders\Lighting.fx"
#endif
#endif

struct STTexCoordLookup
{
	int iTexCoordSet;
	float4x4 TexTransform;
};

struct STScreenCoordLookup
{
	float fBumpStrength;
	float4x4 CoordTransfrom;
};

struct STNormalLookup
{
      float3 Origin;
      float FarPlane;
};

#define TEXMAPDEF(mapname)				\
float g_##mapname##Strength;			\
texture g_##mapname##;					\
sampler g_s##mapname## = sampler_state { \
	texture = <g_##mapname##>;			\
	AddressU = Wrap;					\
	AddressV = Wrap;					\
	AddressW = Wrap;					\
	MipFilter = LINEAR;					\
	MagFilter = LINEAR;					\
	MinFilter = (g_TexFilter);			\
	MaxAnisotropy = (g_TexAnisotropy);	\
};

float4 DoTexCoordLookup(uniform sampler s, uniform STTexCoordLookup param, const float2 texc[NUMTEXCOORDSETS], const float2 texcorrdoffset)
{
//	return tex2D(s, texc[param.iTexCoordSet] + texcorrdoffset);
	return tex2D(s, mul(float4(texc[param.iTexCoordSet] + texcorrdoffset, 0, 1), param.TexTransform).xy);
}

float4 DoScreenCoordLookup(uniform sampler s, uniform STScreenCoordLookup param)
{
	return 0;
}

//#define RAYTRACEDREF

static const int RAYTRACEITERATIONS = 0;

//from: Approximate Ray-Tracing on the GPU with Distance Impostors (EG05)
float3 RaytraceCUBEHit(uniform sampler s, const float3 Point, const float3 Direction)
{
	float dist = texCUBE(s, Direction).a; //distance to first hit -> assume a plane there
	float dp = dist - dot(Point, Direction); //distance fromt actual point to plane at dist
	float3 pp = Point + Direction * dp; //point at this first plane
	//assume a second plane from first hit to second hit
	float ppp = length(pp) / texCUBE(s, pp).a; //ratio of distance to first and second plane 

	float dun = 0;
	float dov = 0;
	float pun = ppp;
	float pov = ppp;

	if (ppp < 1) // second hit was more distant
		dun = dp; 
	else 
		dov = dp;

	float dl = max(dp + dist * (1 - ppp), 0); //euqation 4, distance from Point to approximated hit

	float3 l = Point + Direction * dl;

	// iteration
	for (int i = 0; i < RAYTRACEITERATIONS; i++) 
	{
		float ddl;
		float llp = length(l) / texCUBE(s, l).a;

		if (llp < 1) // undershooting
		{ 
			dun = dl; 
			pun = llp;
			ddl = (dov == 0) ? dist * (1 - llp) : (dl-dov) * (1-llp)/(llp-pov);
		} 
		else // overshooting
		{ 
			dov = dl; 
			pov = llp;
			ddl = (dun == 0) ? dist * (1 - llp) : (dl-dun) * (1-llp)/(llp-pun);
		}

		dl = max(dl + ddl, 0); // avoid flip
		l = Point + Direction * dl;
	}

	return l;
}

float4 DoNormalLookup(const sampler s, uniform STNormalLookup param, const float3 direction, const float3 WorldPos)
{
#ifdef RAYTRACEDREF
	return texCUBE(s, RaytraceCUBEHit(s, (WorldPos - param.Origin) / param.FarPlane, direction));
#else
	return texCUBE(s, direction);
#endif
}

#define LOOKUPFUNCTIONHEAD(mapname) float4 Do##mapname##Lookup(const float2 texc[NUMTEXCOORDSETS], const float2 texcoordoffset, const float3 Direction, const float3 WorldPos)
#define TEXCOORDLOOKUP(mapname) return DoTexCoordLookup(g_s##mapname##, g_##mapname##LookupParam, texc, texcoordoffset);
#define SCREENCOORDLOOKUP(mapname) return DoScreenCoordLookup(g_s##mapname##, g_##mapname##LookupParam);
#define NORMALLOOKUP(mapname) return DoNormalLookup(g_s##mapname##, g_##mapname##LookupParam, Direction, WorldPos);

#ifdef COLORMAP
COLORMAPLOOKUP g_ColorMapLookupParam;
TEXMAPDEF(ColorMap);
LOOKUPFUNCTIONHEAD(ColorMap)
{
#if COLORMAPLOOKUPTYPE == MTLT_TEXCOORD
	TEXCOORDLOOKUP(ColorMap);
#elif COLORMAPLOOKUPTYPE == MTLT_SCREENCOORD
	SCREENCOORDLOOKUP(ColorMap);
#elif COLORMAPLOOKUPTYPE == MTLT_SURFACENORMAL
	NORMALLOOKUP(ColorMap);
#else 
	return 0;
#endif
}
#endif

#ifdef SPECULARMAP
SPECULARMAPLOOKUP g_SpecularMapLookupParam;
TEXMAPDEF(SpecularMap);
LOOKUPFUNCTIONHEAD(SpecularMap)
{
#if SPECULARMAPLOOKUP == MTLT_TEXCOORD
	TEXCOORDLOOKUP(SpecularMap);
#elif SPECULARMAPLOOKUP == MTLT_SCREENCOORD
	SCREENCOORDLOOKUP(SpecularMap);
#elif SPECULARMAPLOOKUP == MTLT_SURFACENORMAL
	NORMALLOOKUP(SpecularMap);
#else 
	return 0;
#endif
}
#endif

#ifdef EMISSIVEMAP
EMISSIVEMAPLOOKUP g_EmissiveMapLookupParam;
TEXMAPDEF(EmissiveMap);
LOOKUPFUNCTIONHEAD(EmissiveMap)
{
#if EMISSIVEMAPLOOKUPTYPE == MTLT_TEXCOORD
	TEXCOORDLOOKUP(EmissiveMap);
#elif EMISSIVEMAPLOOKUPTYPE == MTLT_SCREENCOORD
	SCREENCOORDLOOKUP(EmissiveMap);
#elif EMISSIVEMAPLOOKUPTYPE == MTLT_SURFACENORMAL
	NORMALLOOKUP(EmissiveMap);
#else 
	return 0;
#endif
}
#endif

#ifdef NORMALMAP
NORMALMAPLOOKUP g_NormalMapLookupParam;
TEXMAPDEF(NormalMap);
LOOKUPFUNCTIONHEAD(NormalMap)
{
#if NORMALMAPLOOKUPTYPE == MTLT_TEXCOORD
	TEXCOORDLOOKUP(NormalMap);
#elif NORMALMAPLOOKUPTYPE == MTLT_SCREENCOORD
	SCREENCOORDLOOKUP(NormalMap);
#elif NORMALMAPLOOKUPTYPE == MTLT_SURFACENORMAL
	NORMALLOOKUP(NormalMap);
#else 
	return 0;
#endif
}
#endif

#ifdef HEIGHTMAP
HEIGHTMAPLOOKUP g_HeightMapLookupParam;
TEXMAPDEF(HeightMap);
LOOKUPFUNCTIONHEAD(HeightMap)
{
#if HEIGHTMAPLOOKUPTYPE == MTLT_TEXCOORD
	TEXCOORDLOOKUP(HeightMap);
#elif HEIGHTMAPLOOKUPTYPE == MTLT_SCREENCOORD
	SCREENCOORDLOOKUP(HeightMap);
#elif HEIGHTMAPLOOKUPTYPE == MTLT_SURFACENORMAL
	NORMALLOOKUP(HeightMap);
#else 
	return 0;
#endif
}
#endif

#ifdef REFLECTIONMAP
REFLECTIONMAPLOOKUP g_ReflectionMapLookupParam;
TEXMAPDEF(ReflectionMap);
LOOKUPFUNCTIONHEAD(ReflectionMap)
{
#if REFLECTIONMAPLOOKUPTYPE == MTLT_TEXCOORD
	TEXCOORDLOOKUP(ReflectionMap);
#elif REFLECTIONMAPLOOKUPTYPE == MTLT_SCREENCOORD
	SCREENCOORDLOOKUP(ReflectionMap);
#elif REFLECTIONMAPLOOKUPTYPE == MTLT_SURFACENORMAL
	NORMALLOOKUP(ReflectionMap);
#else 
	return 0;
#endif
}
#endif

#ifdef REFRACTIONMAP
REFRACTIONMAPLOOKUP g_RefractionMapLookupParam;
TEXMAPDEF(RefractionMap);
LOOKUPFUNCTIONHEAD(RefractionMap)
{
#if REFRACTIONMAPLOOKUPTYPE == MTLT_TEXCOORD
	TEXCOORDLOOKUP(RefractionMap);
#elif REFRACTIONMAPLOOKUPTYPE == MTLT_SCREENCOORD
	SCREENCOORDLOOKUP(RefractionMap);
#elif REFRACTIONMAPLOOKUPTYPE == MTLT_SURFACENORMAL
	NORMALLOOKUP(RefractionMap);
#else 
	return 0;
#endif
}
#endif


struct VS_IN
{
    float3 Pos : POSITION;
	//float4 Color : COLOR;
	float3 Normal : NORMAL;
	float3 Tangent : TANGENT;
#if NUMTEXCOORDSETS > 0
    float2 TexC[NUMTEXCOORDSETS] : TEXCOORD0;
#endif
};

struct VS_OUT
{
	float4 Pos : POSITION;
    float4 PosT : TEXCOORD0;

#ifdef LIGHTING
	float3 Normal : TEXCOORD1;
#if MAPPINGTYPE == MT_NORMAL || MAPPINGTYPE == MT_BUMP || MAPPINGTYPE == MT_PARALLAX
	float3 Tangent : TEXCOORD2;
#endif
#endif

#if NUMTEXCOORDSETS > 0
	float2 TexC[NUMTEXCOORDSETS] : TEXCOORD3;
#endif

#ifdef SCATTERING
	float3 Extinction : TEXCOORD6;
	float3 Inscatter : TEXCOORD7;
#endif
//float3 ViewVec : TEXCOORD7;
};

VS_OUT VS(VS_IN i)
{
	VS_OUT o;

	float4 ipos = float4(i.Pos, 1);
	float3 tangent = i.Tangent;
	float3 normal = i.Normal;

#ifdef FRAMEHIERARCHY
	ipos = mul(ipos, g_matFrame);
	normal = mul(normal, g_matFrame);
	tangent = mul(tangent, g_matFrame);
#endif
	
	float4 WorldPos = mul(ipos, g_matWorld);

	o.Pos = mul(WorldPos, g_matVP);

#if NUMTEXCOORDSETS > 0
      [unroll] for(int j = 0; j < NUMTEXCOORDSETS; j++)
	    o.TexC[j] = i.TexC[j];
#endif

    o.PosT = o.Pos;

#ifdef LIGHTING

#if MAPPINGTYPE == MT_NORMAL || MAPPINGTYPE == MT_BUMP || MAPPINGTYPE == MT_PARALLAX
	o.Tangent = normalize(mul(tangent, g_matWorld));
#endif

	o.Normal = normalize(mul(normal, g_matWorld));
#endif

//o.ViewVec = normalize(g_CamPos - WorldPos.xyz);

#ifdef SCATTERING
	STInscatter ins = Inscatter(WorldPos.xyz, g_CamPos);
	o.Inscatter = ins.Inscatter;
	o.Extinction = ins.Extinction;
#endif

	return o;
}

float4 PS(VS_OUT i, uniform int dirlights, uniform int spotlights, uniform int pointlights) : COLOR
{
	//float2 texc = do something bumpyness;
//	float2 texc = i.TexC;
 
	float3 iDiffuse = 0;
	float3 iSpecular = 0;

      float4 v = mul(i.PosT, g_matInvVP);
	float3 WorldPos = v.xyz/v.w;

#ifdef LIGHTING
   

//      float3 ViewVec = normalize(g_CamPos - WorldPos);
	//float3 ViewVec = normalize(i.ViewVec);
	float3 ViewVec = g_CamPos - WorldPos;
      float CamDist = length(ViewVec);
      ViewVec /= CamDist;

//return float4(abs(ViewVec - ViewVec2), 1);

 //return dot(i.TBNMatrix[2], float3(0,1,0));

	
#if MAPPINGTYPE == MT_NORMAL || MAPPINGTYPE == MT_BUMP || MAPPINGTYPE == MT_PARALLAX
	float3 nn = normalize(i.Normal);
	float3 nt = normalize(i.Tangent);
	float3x3 TBNMatrix = float3x3(nt, cross(nt, nn), nn);
#endif

#if MAPPINGTYPE == MT_PARALLAX
      float fHeight = DoHeightMapLookup(i.TexC, float2(0, 0), i.Normal, WorldPos);
	/*#if HEIGHTMAPLOOKUPTYPE == MTLT_TEXCOORD
		float fHeight = DoTexCoordLookup(g_sHeightMap, g_HeightMapLookupParam, i.TexC, float2(0, 0)) * 2 - 1;
	#elif HEIGHTMAPLOOKUPTYPE == MTLT_SCREENCOORD
		float fHeight = DoScreenCoordLookup(g_sHeightMap, g_HeightMapLookupParam) * 2 - 1;
	#else
		float fHeight = DoNormalLookup(g_sHeightMap, g_HeightMapLookupParam, i.Normal, WorldPos) * 2 - 1;
	#endif*/
	float3 ViewVecTang = mul(TBNMatrix, ViewVec);
	float2 TexOffset = g_HeightMapStrength * (fHeight - 0.5) * ViewVecTang * 0.01f;
#else
	float2 TexOffset = 0;
#endif

	 //get tangent-space normal
#if MAPPINGTYPE == MT_NORMAL || MAPPINGTYPE == MT_BUMP || MAPPINGTYPE == MT_PARALLAX
      float3 Normal = DoNormalMapLookup(i.TexC, float2(0, 0), i.Normal, WorldPos).rgb * 2 - 1;
	/*#if NORMALMAPLOOKUPTYPE == MTLT_TEXCOORD
		float3 Normal = DoTexCoordLookup(g_sNormalMap, g_NormalMapLookupParam, i.TexC, TexOffset) * 2 - 1;
	#elif NORMALMAPLOOKUPTYPE == MTLT_SCREENCOORD
		float3 Normal = DoScreenCoordLookup(g_sNormalMap, g_NormalMapLookupParam) * 2 - 1;
	#else
		float3 Normal = DoNormalLookup(g_sNormalMap, g_NormalMapLookupParam, i.Normal, WorldPos) * 2 - 1;
	#endif*/

	//Normal = float3(0, 0, 1);
	//transform back to worldspace where lighting is performed
	//mul can be done, because TBN is assumed to be orthonormal
	Normal = mul(Normal, TBNMatrix);
#else 
	float3 Normal = normalize(i.Normal);
#endif

//	return float4(Normal * 0.5f + 0.5f, 1);
	//return float4(Normal, 1);

    //Normal = float3(0, 0, 1);
//Normal = - Normal;
	
	[unroll] for(int l=0; l < dirlights; l++)
	{
		DoDirLight(l, WorldPos, Normal, ViewVec, CamDist, g_Material.SpecularPower, iDiffuse, iSpecular);
	}
//return float4(i.TBNMatrix[2],1);

	[unroll] for(int l=0; l < spotlights; l++)
	{
		DoSpotLight(l, WorldPos, Normal, ViewVec, g_Material.SpecularPower, iDiffuse, iSpecular);
	}

	[unroll] for(int l=0; l < pointlights; l++)
	{
		DoPointLight(l, WorldPos, Normal, ViewVec, g_Material.SpecularPower, iDiffuse, iSpecular);
	}
	      
#endif //LIGHTING

	float3 amb = g_Material.AmbientColor * g_AmbientLight.rgb;// * g_AmbientLightStrength.a;
	float3 dif = g_Material.DiffuseColor * iDiffuse;
	float3 spec = g_Material.SpecularColor * iSpecular;
	float3 emi = g_Material.EmissiveColor * g_Material.EmissiveStrength;
	float opacity = g_Material.Opacity;

#ifdef COLORMAP
	float4 ColorMap = DoColorMapLookup(i.TexC, TexOffset, Normal, WorldPos);
/*	#if COLORMAPLOOKUPTYPE == MTLT_TEXCOORD
		float4 ColorMap = DoTexCoordLookup(g_sColorMap, g_ColorMapLookupParam, i.TexC, TexOffset);
	#elif COLORMAPLOOKUPTYPE == MTLT_SCREENCOORD
		float4 ColorMap = DoScreenCoordLookup(g_sColorMap, g_ColorMapLookupParam);
	#else
		float4 ColorMap = DoNormalLookup(g_sColorMap, g_ColorMapLookupParam);
	#endif*/

		amb *= ColorMap.rgb;
		dif *= ColorMap.rgb;
		opacity *= ColorMap.a;
#endif

#ifdef SPECULARMAP
      spec *= DoSpecularMapLookup(i.TexC, TexOffset, Normal, WorldPos).rgb;
/*	#if SPECULARMAPLOOKUPTYPE == MTLT_TEXCOORD
		spec * = DoTexCoordLookup(g_sSpecularMap, g_SpecularMapLookupParam, i.TexC, TexOffset);
	#elif SPECULARMAPLOOKUPTYPE == MTLT_SCREENCOORD
		spec * = DoScreenCoordLookup(g_sSpecularMap, g_SpecularMapLookupParam);
	#else
		spec * = DoNormalLookup(g_sSpecularMap, g_SpecularMapLookupParam, Normal, WorldPos);
	#endif*/
#endif

#ifdef EMISSIVEMAP
      emi *= DoEmissiveMapLookup(i.TexC, TexOffset, Normal, WorldPos).rgb;
/*	#if EMISSIVEMAPLOOKUPTYPE == MTLT_TEXCOORD
		emi * = DoTexCoordLookup(g_sEmissiveMap, g_EmissiveMapLookupParam, i.TexC, TexOffset);
	#elif EMISSIVEMAPLOOKUPTYPE == MTLT_SCREENCOORD
		emi * = DoScreenCoordLookup(g_sEmissiveMap, g_EmissiveMapLookupParam);
	#else
		emi * = DoNormalLookup(g_sEmissiveMap, g_EmissiveMapLookupParam, Normal, WorldPos);
	#endif*/
#endif

#ifdef REFLECTIONMAP
	float4 refl = DoReflectionMapLookup(i.TexC, TexOffset, reflect(-ViewVec, Normal), WorldPos);
//	float4 refl = DoReflectionMapLookup(i.TexC, TexOffset, Normal, WorldPos);

/*      #if REFLECTIONMAPLOOKUPTYPE == MTLT_TEXCOORD
		refl = DoTexCoordLookup(g_sReflectionMap, g_ReflectionMapLookupParam, i.TexC, TexOffset);
	#elif REFLECTIONMAPLOOKUPTYPE == MTLT_SCREENCOORD
		refl = DoScreenCoordLookup(g_sReflectionMap, g_ReflectionMapLookupParam);
	#else
		refl = DoNormalLookup(g_sReflectionMap, g_ReflectionMapLookupParam, reflect(-ViewVec, Normal), WorldPos);
	#endif*/
	dif = lerp(dif, refl, g_ReflectionMapStrength);
//      dif = refl;
#endif

#ifdef REFRACTIONMAP
      float4 refr = DoReflectionMapLookup(i.TexC, TexOffset, float3(0,0,0), WorldPos);
#endif

	float3 Color = amb + dif + spec + emi;

#ifdef SCATTERING
	Color = Color * i.Extinction + i.Inscatter;
#endif

//    float far = g_matProj[2][3] / (1 - g_matProj[2][2]);
//	return mul(WorldPos, g_matView).z / far;

#ifdef ALPHAENABLE
	//only write alpha when any alpha func is enabled
	//when also DEPTHTOALPHA is enabled in a second pass will be rendered for
	return float4(Color, opacity);
#else 
	//otherwise return depthvalue in alpha channel
      float far = g_matProj[2][3] / (1 - g_matProj[2][2]);
	float depth = 1 - mul(WorldPos, g_matView).z / far;

	return float4(Color, depth);
#endif
}



void VSShadow(VS_IN i, out float4 oPos : POSITION, out float2 oDepth : TEXCOORD0)
{
	float4 ipos = float4(i.Pos, 1);

#ifdef FRAMEHIERARCHY
	ipos = mul(ipos, g_matFrame);
#endif
	
	float4 WorldPos = mul(ipos, g_matWorld);
    //transform to lightspace, when the shadow map is rendered the VP matrix is equal to the SMMatrix
	oPos = mul(WorldPos, g_matVP);

	oDepth = oPos.zw;
}

float4 PSShadow(float2 iDepth : TEXCOORD0) : COLOR
{
	//TODO: read diffuse texture if alphatesting and do testing too
	return iDepth.x / iDepth.y;
}

stateblock RenderStates = stateblock_state
{
#ifdef DOUBLESIDED
	CullMode = NONE;
#else
	CullMode = g_CullMode;
#endif

    ZEnable = true;
    ZWriteEnable = true;
    ZFunc = LESS;

	ColorWriteEnable = RED | GREEN | BLUE | ALPHA;

#ifdef ALPHAENABLE
	AlphaBlendEnable = g_Blending.AlphaBlendEnable;
	SrcBlend = SRCALPHA;
	DestBlend = INVSRCALPHA;
	BlendOp = ADD;

    AlphaTestEnable = g_Blending.AlphaTestEnable; 
    AlphaRef = g_Blending.AlphaRef;
	AlphaFunc = GREATER;

#ifdef DEPTHTOALPHA
	//dont overwrite depth in alphachannel
	SeparateAlphaBlendEnable = true;
	SrcBlendAlpha = ONE;
	DestBlendAlpha = ZERO;
	BlendOpAlpha = ADD;
#else
	SeparateAlphaBlendEnable = false;
#endif
#else
	AlphaBlendEnable = false;
	AlphaTestEnable = false;
	SeparateAlphaBlendEnable = false;
#endif

	//BlendOp = ADD;
	//BlendOpAlpha = ADD;
};

#ifndef NUMDIR
#define NUMDIR 0
#endif

#ifndef NUMSPOT
#define NUMSPOT 0
#endif

#ifndef NUMPOINT
#define NUMPOINT 0
#endif

technique Render //no lights
{
    pass P0
    {
        StateBlock = (RenderStates);

        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS(NUMDIR, NUMSPOT, NUMPOINT);
    }

#ifdef DEPTHTOALPHA
#ifdef ALPHABLENDING
	//separately render the depth to alpha channel when alphablending or testing is enabled
	//for alphatesting zequal will ensure the correct result
	pass P1
	{
		ZEnable = true;
		ZWriteEnable = false;
		ZFunc = EQUAL;

		ColorWriteEnable = ALPHA;

		AlphaBlendEnable = false;
		AlphaTestEnable = false; 

		//AlphaTestEnable = true; 
		//AlphaRef = g_Blending.AlphaRef;
		//AlphaFunc = GREATER;

		//the view-projection matrix defines shadow map matrix, therefore it can be used to write only the depth to the alpha channel
		VertexShader = compile vs_3_0 VSShadow();
		PixelShader  = compile ps_3_0 PSShadow();
	}
#endif
#endif
}

technique Render_Shadow //render shadowmap
{
    pass P0
    {
        StateBlock = (RenderStates);

        VertexShader = compile vs_3_0 VSShadow();
        PixelShader  = compile ps_3_0 PSShadow();
    }
}
