float4x4 matTot;
float2  BlockOffset;
float   QuadSize;
float3  CamPos;
float3  CamDir;
float3  SunDir;
float2  TexTile; 
float2  NormalStrength;
float   DisplaceStrength;
float   ReflBumpStrength;
float   SunShininess;
float4  SunColor;
float4  OceanColor;
float4  SkyColor;
float   Ambient;
float   Alpha;

float4 g_TexTile = {0.01, 0.08, 0.64, 0.05};
float4 g_NormalStrength = {0.05, 0.2, 0.1, 0.16};

#include "Shaders/Scattering.fx"

int TexFilter;
int TexAnisotropy;

texture NormalMap0, NormalMap1, NormalMap2, NormalMap3;

sampler2D SNormalMap0 = sampler_state
{
	Texture = <NormalMap0>;
	AddressU = Wrap;
    AddressV = Wrap;
    MipFilter = LINEAR;
    MinFilter = (TexFilter);
	MaxAnisotropy = (TexAnisotropy);
};

sampler2D SNormalMap1 = sampler_state
{
	Texture = <NormalMap1>;
	AddressU = Wrap;
    AddressV = Wrap;
    MipFilter = LINEAR;
    MinFilter = (TexFilter);
	MaxAnisotropy = (TexAnisotropy);
};

sampler2D SNormalMap2 = sampler_state
{
	Texture = <NormalMap2>;
	AddressU = Wrap;
    AddressV = Wrap;
    MipFilter = LINEAR;
    MinFilter = (TexFilter);
	MaxAnisotropy = (TexAnisotropy);
};

sampler2D SNormalMap3 = sampler_state
{
	Texture = <NormalMap3>;
	AddressU = Wrap;
    AddressV = Wrap;
    MipFilter = LINEAR;
    MinFilter = (TexFilter);
	MaxAnisotropy = (TexAnisotropy);
};

texture Reflection;
sampler SReflection = sampler_state
{
    Texture = <Reflection>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = LINEAR;
    MinFilter = LINEAR;
};

texture Refraction;
sampler SRefraction = sampler_state
{
    Texture = <Refraction>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = LINEAR;
    MinFilter = LINEAR;
};

struct OCEAN_INPUT
{
    float3 Pos : POSITION;
};

struct OCEAN30_OUTPUT
{
    float4 Pos : POSITION;
    float2 Wave0 : TEXCOORD0;
    float2 Wave1 : TEXCOORD1;
    float3 WPos : TEXCOORD2;
    float3 ScreenPos : TEXCOORD3;
    float3 Normal : TEXCOORD4;
    float3 Tangent : TEXCOORD5;
	float3 Extinction : TEXCOORD6;
	float3 Inscatter : TEXCOORD7;
};

OCEAN30_OUTPUT VSOcean30(OCEAN_INPUT i, uniform bool bRefl, uniform bool bRefr)
{
    OCEAN30_OUTPUT o;

    float dz = 1 - abs(CamDir.z); 
    float cz = abs(CamPos.z) + 0.1;
    float h = cz * 0.00000000000003;
    float ds = 50 + 165 *dz;

    float2 gp = CamPos.xy + i.Pos.xy * pow(i.Pos.z + ds, 6) * h;

    //float3 p = i.Pos * QuadSize + float3(BlockOffset,0);
    
    float3 wpos = float3(gp, 0);

    float4 clip = mul(float4(wpos, 1), matTot);
    float3 b = abs(clip.xyz) < (clip.www * 1.1 + 1.1);

    if(all(b))
    {
        float d = length(CamPos - wpos);

        float lod = sqrt(d * 0.05);

        float4 wavetex0 = tex2Dlod(SNormalMap0, float4(gp.xy * g_TexTile.x,0,lod));

        wpos.z += (wavetex0.a - 0.5f) * DisplaceStrength;

//        float3 normal = wavetex0.rgb * 2 - 1;

        float3 normal = float3(wavetex0.rg * 2 - 1, 1);
		normal.xy *= g_NormalStrength.x;
//        normal.xy *= 0.125;
        normal = normalize(normal);
        float3 binormal = float3(1, 0, 0);

        o.Normal = normal;
        o.Tangent = cross(normal, binormal);

		STInscatter ins = Inscatter(wpos, CamPos);
		o.Inscatter = ins.Inscatter;
		o.Extinction = ins.Extinction;
    }
    else
    {
        o.Normal = 0;
        o.Tangent = 0;
		o.Inscatter = 0;
		o.Extinction = 0;
    }
    //float4 wavetex1 = tex2Dlod(SNormalMap1, float4(gp * TexTile.y,0,0));

    //gp.z += (wavetex0.a - 0.5f) * DisplaceStrength;
    //p.z += wavetex1.a * DisplaceStrength.y;
    
        o.WPos = wpos;
        o.Pos = mul(float4(wpos, 1), matTot);

        o.Wave0 = gp * g_TexTile.y;
        o.Wave1 = gp * g_TexTile.z;
    
        o.ScreenPos = o.Pos.xyz/o.Pos.w;
        o.ScreenPos.xy = 0.5 + o.ScreenPos.xy * float2(0.5,-0.5);
        o.ScreenPos.z = ReflBumpStrength / o.ScreenPos.z;
    
    //o.Tangent.x = i.Pos.z/250;

/*  float3x3 matTangentSpace = {t0, cross(t0, n0), n0};

    float3 n1 = wavetex1.rgb*2-1;
    n1.xy *= TexStrength.y;
    o.Normal = normalize(mul(n1, matTangentSpace));

    float3 b1 = float3(1,0,0);
    o.Tangent = normalize(cross(n1,b1));*/
    
    return o;
}

float4 PSOcean30(OCEAN30_OUTPUT i, uniform bool bRefl, uniform bool bRefr) : COLOR
{
   // float3 n2 = tex2D(SNormalMap[1],i.Wave1).xyz*2-1; //also bias

/*    float3 iNormal = i.Normal;
//    float3 iNormal = (tex2D(SNormalMap[0],i.Wave0).xyz*2-1);
 //   iNormal.xy *= NormalStrength.x;
    iNormal = normalize(iNormal);*/

    float3 Normal;

    if(i.ScreenPos.z > 0.500)
    {
        float3 n = normalize(i.Normal);
        float3 t = normalize(i.Tangent);
        float3 b = cross(t, n);

        float3x3 matTangentSpace = {t, b, n};

        float2 n1 = (tex2D(SNormalMap1, i.Wave0).xy * 2 - 1) * g_NormalStrength.y * 0.5;
        float2 n2 = (tex2D(SNormalMap2, i.Wave1).xy * 2 - 1) * g_NormalStrength.z * 0.5;

//        Normal = normalize(i.Normal);
//        Normal = normalize(float3(n2.xy,1));

        Normal = mul(normalize(float3(n1 + n2, 1)), matTangentSpace);

/*        float3 n2 = tex2D(SNormalMap[1],i.Wave1).xyz*2-1; //also bias
        n2.xy *= NormalStrength.y;
        Normal = normalize(mul(n2, matTangentSpace));   
        if(i.ScreenPos.z < 0.510)
        {
            float s = (i.ScreenPos.z - 0.500)/0.01;
            Normal = lerp(iNormal, Normal, 0);
        }*/   
    }
    else
    {
        Normal = i.Normal;
    }

//    return float4(abs(Normal.xy),0,1);

/*    float3 Normal = tex2D(SNormalMap[0],i.Wave0).xyz*2-1;
    Normal.xy *= NormalStrength.x*3;
    Normal = normalize(Normal);
//    float3 Normal = float3(0, 0, 1);
    Normal.xy += (tex2D(SNormalMap[1],i.Wave1).xy*2-1) * NormalStrength.y*3;
    Normal = normalize(Normal);*/

    float3 CamDir = normalize(CamPos - i.WPos);
    float3 ReflVec = reflect(-CamDir,Normal);

    float Fresnel = 1.f/pow(saturate(dot(ReflVec,Normal))+1,8) * 0.4 + 0.6;

    float3 ReflColor = 0;
    float ReflDist = 1;

    if(bRefl)
	{
        float4 vtex = tex2D(SReflection,i.ScreenPos.xy - i.ScreenPos.z * Normal.xy * 0.2);
        //float4 vtex = tex2D(SReflection,i.ScreenPos.xy);
        ReflColor = vtex.rgb;
        ReflDist = vtex.a;
	}
    
    float AmbientColor = Ambient;

    float4 OColor = OceanColor + Ambient;

    //float x = dot(Normal,  float3(0.4,0.4,0.4));
    //float4 OColor = min(x,AmbientColor) * OceanColor;
    //float4 OColor = OceanColor * (dot(SkyColor, float3(0.333,0.333,0.333))*0.3 + x)  + AmbientColor;
//    float4 OColor = OceanColor * (dot(SkyColor, float3(0.333,0.333,0.333))*0.3)  + AmbientColor;
    
    float3 RefrColor = OColor;
	float RefrDist = 1;
    if(bRefr)
    {
		float4 vtex = tex2D(SRefraction,i.ScreenPos.xy + i.ScreenPos.z * Normal.xy * 0.05);
        RefrColor = vtex.rgb;
		RefrDist = vtex.a;
    }
    
    float fSpec;
    fSpec = pow(saturate(dot(ReflVec, SunDir)), 32) * 1;
    fSpec += pow(saturate(dot(ReflVec, SunDir)), 512) * 3;

    float4 SunSpecular = fSpec * SunColor;

    SunSpecular *= ReflDist < 1 ? 0 : 1;

//return SunSpecular;

//  return RefrColor.a;

    //return ReflColor;
    //return i.Tangent.x;

//    float3 Color = lerp(SkyColor,ReflColor.rgb,Refl.a);
    float3 Color = lerp(RefrColor,ReflColor,Fresnel);

	Color.rgb = Color * i.Extinction + i.Inscatter;

    if(bRefr)
        return float4(Color + SunSpecular, 1);
    else
        return float4(Color + SunSpecular, Alpha);

    //return float4(AmbientColor,AmbientColor,AmbientColor,1);
    
    /*float x = Normal.x;
    float3 c = float3(x,x,x);
    return float4(c,1);*/
}

stateblock RenderStatesBlend = stateblock_state
{
    ZEnable = true;
    ZWriteEnable = true;
    ZFunc = LESS;

    AlphaBlendEnable = true;
    AlphaTestEnable = false;
    BlendOp = ADD;
    SrcBlend = SRCALPHA;
    DestBlend = INVSRCALPHA;
};

stateblock RenderStatesNoBlend = stateblock_state
{
    ZEnable = true;
    ZWriteEnable = true;
    ZFunc = LESS;

    AlphaBlendEnable = false;
    AlphaTestEnable = false;
};

technique OceanGrid
{
    pass P0
    {          
        StateBlock = (RenderStatesBlend);
                
        VertexShader = compile vs_3_0 VSOcean30(false, false);
        PixelShader  = compile ps_3_0 PSOcean30(false, false);
    }
}

technique OceanGrid_Refl
{
    pass P0
    {          
        StateBlock = (RenderStatesBlend);
                
        VertexShader = compile vs_3_0 VSOcean30(true, false);
        PixelShader  = compile ps_3_0 PSOcean30(true, false);
    }
}

technique OceanGrid_Refl_Refr
{
    pass P0
    {          
        StateBlock = (RenderStatesNoBlend);
                
        VertexShader = compile vs_3_0 VSOcean30(true, true);
        PixelShader  = compile ps_3_0 PSOcean30(true, true);
    }
}














