#include "Shaders/Lighting.fx"
#include "Shaders/Scattering.fx"

float4x4    matViewProj : VIEWPROJ;
float4x4	g_matWorld;
float4x4	g_matInvVP : INVVIEWPROJ;
float4x4    matView : VIEW;
float4x4    matProj : PROJ;
float3      vLightDir;
float4      LightColor;
float4      g_AmbientLight : AMBIENT;
float3      g_CamPos : CAMPOS;
float       Scale;
float4      LightInd;

float4      LeavesAnimTable[30];

int TexFilter : TEXFILTER;
int TexAnisotropy : TEXANISO;

texture Texture;
sampler STexture = sampler_state
{
    Texture = <Texture>;
    AddressU = Wrap;
    AddressV = Wrap;
    MipFilter = LINEAR;
    MagFilter = LINEAR;
    MinFilter = (TexFilter);
    MaxAnisotropy = (TexAnisotropy);
};

struct VS_FRONDS_INPUT
{
    float3 Pos : POSITION;
    float2 Tex : TEXCOORD0;
};

struct VS_LEAVES_INPUT
{
    float3 Pos : POSITION;
    float4 Color : COLOR0; //w has index for animation
    float2 Expansion : TEXCOORD0;
    float4 Dimm : COLOR1;
};

struct VS_OUTPUT
{
    float4 Pos : POSITION;
    float4 Diffuse : COLOR0;
    float2 Tex : TEXCOORD0;
	float4 PosT : TEXCOORD1;
	float3 Extinction : TEXCOORD2;
	float3 Inscatter : TEXCOORD3;
};

struct VS_SHADOW_OUTPUT
{
    float4 Pos : POSITION;
    float2 Depth : TEXCOORD0;
};

struct VS_ALPHASHADOW_OUTPUT
{
    float4 Pos : POSITION;
    float2 Depth : TEXCOORD0;
    float2 Tex : TEXCOORD1;
};

VS_OUTPUT FrondVS(VS_FRONDS_INPUT i)
{
    VS_OUTPUT o;
    
	float4 wpos = mul(float4(i.Pos, 1), g_matWorld);

    o.Pos = mul(wpos, matViewProj);
    
    o.Tex = i.Tex;
        
    //o.Diffuse = LightColor;
	o.Diffuse = 1;

	o.PosT = o.Pos;

	STInscatter ins = Inscatter(wpos.xyz, g_CamPos);
	o.Inscatter = ins.Inscatter;
	o.Extinction = ins.Extinction;
        
    return o;
}

VS_ALPHASHADOW_OUTPUT FrondShadowVS(VS_FRONDS_INPUT i)
{
    VS_ALPHASHADOW_OUTPUT o;

    //o.Pos = mul( mul(float4(i.Pos,1), g_matWorld), SM_matSM);
	o.Pos = mul( mul(float4(i.Pos,1), g_matWorld), matViewProj);
    
    o.Tex = i.Tex;

    o.Depth.xy = o.Pos.zw;
    
    return o;
}

VS_OUTPUT LeafVS(VS_LEAVES_INPUT i)
{
    VS_OUTPUT o;
     
    int animind = i.Color.w*256;
    
    float4 wpos = mul(float4(i.Pos,1), g_matWorld);
	float4 p = mul(wpos, matView);
    
    p.xyz += i.Expansion.x * LeavesAnimTable[animind].xyz * Scale;
    p.xyz += i.Expansion.y * -LeavesAnimTable[animind].wxz * Scale;
    
    o.Pos = mul(p, matProj);

    o.Tex = saturate(sign(i.Expansion));

    //lightind stuff
    float light = i.Dimm.x*LightInd.x + i.Dimm.y*LightInd.y + i.Dimm.z*LightInd.z + i.Dimm.w*LightInd.w;
    
    //o.Diffuse.xyz = LightColor * i.Color * light * light * light;
    o.Diffuse.xyz = i.Color * light * light * light;

    o.Diffuse.w = 1;

	o.PosT = o.Pos;

	STInscatter ins = Inscatter(wpos.xyz, g_CamPos);
	o.Inscatter = ins.Inscatter;
	o.Extinction = ins.Extinction;
            
    return o;
}

VS_ALPHASHADOW_OUTPUT LeafShadowVS(VS_LEAVES_INPUT i)
{
    VS_ALPHASHADOW_OUTPUT o;

    int animind = i.Color.w*256;
    
    float4 wpos = mul(float4(i.Pos,1), g_matWorld);
	float4 p = mul(wpos, matView);
    
    p.xyz += i.Expansion.x * Scale * LeavesAnimTable[animind].xyz;
    p.xyz += i.Expansion.y * Scale * -LeavesAnimTable[animind].wxz;
    
    o.Pos = mul(p, matProj);
    
    o.Tex = saturate(sign(i.Expansion));

    o.Depth.xy = o.Pos.zw;
    
    return o;
}

float4 PS(VS_OUTPUT i, uniform int dirlights, uniform int spotlights, uniform int pointlights) : COLOR
{
    float4 DiffuseColor = tex2D(STexture,i.Tex);

	float4 v = mul(i.PosT, g_matInvVP);
	float3 WorldPos = v.xyz/v.w;

	float4 iDiffuse = 0;

	[unroll] for(int l=0; l < dirlights; l++)
	{
		iDiffuse += g_DirLights[l].Color;
	}

	[unroll] for(int l=0; l < spotlights; l++)
	{
		float LightDist = length(g_SpotLights[l].Position - WorldPos);	

		float3 LightVecWorld = normalize(g_SpotLights[l].Position - WorldPos);

		float Att = LightAttenuation(LightDist, g_SpotLights[l].Attenuation);

		//spot open angle
		float rho = dot(LightVecWorld, -g_SpotLights[l].Direction);
		//apply attenuation depending on open angle
		Att *= pow( saturate( (rho - g_SpotLights[l].SpotParam.y) / (g_SpotLights[l].SpotParam.x - g_SpotLights[l].SpotParam.y) ), g_SpotLights[l].SpotParam.z );

		iDiffuse += g_SpotLights[l].Color * Att;
	}

	[unroll] for(int l=0; l < pointlights; l++)
	{
		//distance to light
		float LightDist = length(g_PointLights[l].Position - WorldPos);

		//calculate light attenuation
		float Att = LightAttenuation(LightDist, g_PointLights[l].Attenuation);

		iDiffuse += g_SpotLights[l].Color * Att;
	}

	float3 Color = (g_AmbientLight.rgb + iDiffuse.rgb * i.Diffuse) * DiffuseColor.rgb;

	Color.rgb = Color * i.Extinction + i.Inscatter;

    return float4(Color, DiffuseColor.a);
}

float4 ShadowPS(VS_SHADOW_OUTPUT i) : COLOR
{
    return i.Depth.x/i.Depth.y;
}

float4 AlphaShadowPS(VS_ALPHASHADOW_OUTPUT i) : COLOR
{
    float ta = tex2D(STexture, i.Tex).a;
//    float4 ta = float4(1,1,1,tex2D(STexture,i.Tex).a);
//    clip(ta-float4(0,0,0,0.3));
    return float4(i.Depth.x/i.Depth.y, 0, 0, ta);
}

stateblock RSSolid = stateblock_state
{
	AlphaBlendEnable = false;
	AlphaTestEnable = false; 
	ZEnable = TRUE;
	ZFunc = LESS;
	Cullmode = CCW; 
	ZWriteEnable = true;
};

stateblock RSAlpha = stateblock_state
{
	AlphaBlendEnable = false;
	AlphaTestEnable = true; 
	AlphaRef = 0x30;
	AlphaFunc = GREATER;
	ZEnable = TRUE;
	ZFunc =  LESS;
	Cullmode = NONE; 
	ZWriteEnable = true;
};

#ifndef NUMDIR
#define NUMDIR 0
#endif

#ifndef NUMSPOT
#define NUMSPOT 0
#endif

#ifndef NUMPOINT
#define NUMPOINT 0
#endif

technique RenderFronds
{
    pass P0
    {
		StateBlock = (RSAlpha);

        VertexShader = compile vs_3_0 FrondVS();
        PixelShader  = compile ps_3_0 PS(NUMDIR, NUMSPOT, NUMPOINT);
    }
}

technique RenderFronds_ShadowMap
{
    pass P0
    {
		StateBlock = (RSAlpha);

        VertexShader = compile vs_3_0 FrondShadowVS();
        PixelShader  = compile ps_3_0 AlphaShadowPS();
    }
}

technique RenderLeaves
{
    pass P0
    {
		StateBlock = (RSAlpha);

        VertexShader = compile vs_2_0 LeafVS();
        PixelShader  = compile ps_2_0 PS(NUMDIR, NUMSPOT, NUMPOINT);
    }
}

technique RenderLeaves_ShadowMap
{
    pass P0
    {
		StateBlock = (RSAlpha);

        VertexShader = compile vs_3_0 LeafShadowVS();
        PixelShader  = compile ps_3_0 AlphaShadowPS();
    }
}



