float4x4 matWorld;

#include "Shaders/Lighting.fx"
#include "Shaders/Scattering.fx"

float4x4    g_matVP : VIEWPROJ;
float4x4	g_matInvVP : INVVIEWPROJ;
float2      g_BlockOffset;
float3      g_CamPos : CAMPOS;

float       g_RoadWidth;
float3      g_ABL; //a, b, length
float2      g_uStartRange;

float       g_AmbientLight : AMBIENT;

float       g_BlockSize;
float       g_FieldSize;
//float2      Start;

//bounds of segment in segment coordinates
float4      g_SegYminStartEnd;
float4      g_SegYmaxStartEnd;

int         g_NumSegments;

float4x4    g_matSegToBlock;
float4x4    g_matBlockToSeg;

int g_TexFilter : TEXFILTER;
int g_TexAnisotropy : TEXANISO;
int g_CullMode : CULLMODE;

#define ZBIAS 0.03

texture g_Texture;
sampler g_sTex = sampler_state
{
    Texture = <g_Texture>;
    AddressU = Wrap;
    AddressV = Wrap;
    MipFilter = LINEAR;
    MagFilter = LINEAR;
    MinFilter = (g_TexFilter);
    MaxAnisotropy = (g_TexAnisotropy);
};

texture g_HeightMap;
sampler g_sHeightMap = sampler_state
{
    Texture = <g_HeightMap>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = LINEAR;
    MinFilter = LINEAR;
};

texture g_NormalMap;
sampler g_sNormalMap = sampler_state
{
    Texture = <g_NormalMap>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = LINEAR;
    MinFilter = LINEAR;
};

struct CURVE_INPUT
{
    float2 Pos : POSITION;
 //   float3 Normal : NORMAL;
//    float2 CurvePos : TEXCOORD0;
};

struct CURVE_OUTPUT
{
    float4 Pos : POSITION;
    float2 CurvePos : TEXCOORD0;
    float3 AlphaBetaGamma : TEXCOORD1;
    float2 BlockPos : TEXCOORD2; //2d world coordinates in block
	float4 PosT : TEXCOORD3;
	float3 Normal : TEXCOORD4;
	float3 Extinction : TEXCOORD5;
	float3 Inscatter : TEXCOORD6;
//float2 h: TEXCOORD5;
};

float fx(const float x)
{ 
    return x*(1-x)*(1-x)*g_ABL.x - x*x*(1-x)*g_ABL.y;
}

float dfx(const float x)
{
    return 3*x*x*(g_ABL.x + g_ABL.y) - 2*x*(2*g_ABL.x + g_ABL.y) + g_ABL.x;
}

CURVE_OUTPUT CurveVS(CURVE_INPUT i)
{
    CURVE_OUTPUT o;

    float2 ipseg = i.Pos;
    ipseg.x /= g_NumSegments;

    const float2 SegStart = lerp(g_SegYminStartEnd.xy, g_SegYmaxStartEnd.xy, ipseg.y);
    const float2 SegEnd = lerp(g_SegYminStartEnd.zw, g_SegYmaxStartEnd.zw, ipseg.y);
    const float2 SegPos = lerp(SegStart, SegEnd, ipseg.x);
    const float2 BlockPos = mul(float4(SegPos,0,1), g_matSegToBlock);
    const float2 WorldPos = BlockPos * g_FieldSize + g_BlockOffset;

    const float Alpha = 3 * SegPos.y * (g_ABL.x + g_ABL.y);
    const float Beta = -1 -(4 * g_ABL.x + 2 * g_ABL.y) * SegPos.y;
    const float Gamma = SegPos.x + SegPos.y * g_ABL.x;

    const float u = 2 * Gamma / (sqrt(Beta * Beta - 4 * Alpha * Gamma) - Beta);

    const float2 BlockPosV0 = mul(float4(u, fx(u), 0, 1), g_matSegToBlock);

    const float2 hmtc = (BlockPosV0 + 1.5) / (g_BlockSize + 3);
    const float2 nmtc = (BlockPosV0 + 0.5) / (g_BlockSize + 1);

    const float Z = tex2Dlod(g_sHeightMap, float4(hmtc, 0, 0));

    const float2 nxy = tex2Dlod(g_sNormalMap, float4(nmtc, 0, 0)) * 2 - 1;
    const float3 Normal = float3(nxy, 1 - nxy.x*nxy.x - nxy.y*nxy.y); 

    const float outz = Z + ZBIAS;

    float4 opos = float4(WorldPos, outz, 1);
    
    o.Pos = mul(opos, g_matVP);

    //shift z to avoid zfighting
    //o.Pos.z -= 0.005 + 0.5 / o.Pos.w;
    
    o.CurvePos = SegPos;
    
    o.AlphaBetaGamma = float3(Alpha, Beta, Gamma);
    
    o.Normal = Normal;

    o.BlockPos = BlockPos;
//    o.BlockPos = BlockPosV0;
//    o.BlockPos = abs(float2(u, fx(u)));

//o.h = BlockPosV0;

    o.PosT = o.Pos;

	STInscatter ins = Inscatter(opos.xyz, g_CamPos);
	o.Inscatter = ins.Inscatter;
	o.Extinction = ins.Extinction;
        
    return o;
}

float4 CurvePS(CURVE_OUTPUT i, uniform int dirlights, uniform int spotlights, uniform int pointlights) : COLOR
{
//return float4(abs(i.BlockPos - i.h),0, 1);
    clip(float4(i.BlockPos, -i.BlockPos + g_BlockSize));

    const float Alpha = i.AlphaBetaGamma.x;
    const float Beta = i.AlphaBetaGamma.y;
    const float Gamma = i.AlphaBetaGamma.z;

    float u = 2 * Gamma / (sqrt(Beta * Beta - 4 * Alpha * Gamma) - Beta);

    //clip(float2(u, -u+1));
    
    float fu = fx(u);
    float dfu = dfx(u);
    
    float2 CurveOffset = i.CurvePos.xy - float2(u,fu);
    float2 CurveNormal = float2(-dfu,1);
    
    float v = dot(CurveNormal, CurveOffset)/sqrt(dfu*dfu+1);

    v = v * g_ABL.z/g_RoadWidth + 0.5;
    clip(float2(v,-v+1));

    float2 texcoord = float2(g_uStartRange.x + u * g_uStartRange.y, v);
    
	float4 ColorMap = tex2D(g_sTex, texcoord);

	float4 unproj = mul(i.PosT, g_matInvVP);
	float3 WorldPos = unproj.xyz / unproj.w;

	float3 iDiffuse = 0;
	float3 iSpecular = 0;

	float3 Tangent = mul(float3(CurveNormal, 0), g_matSegToBlock);
	float3 Binormal = cross(Tangent, i.Normal);

	float3x3 TBNMatrix = float3x3(Tangent, Binormal, i.Normal);

	float3 ViewVec = g_CamPos - WorldPos;
      float CamDist = length(ViewVec);
      ViewVec /= CamDist;

	float3 Normal = TBNMatrix[2];

	[unroll] for(int l=0; l < dirlights; l++)
	{
		DoDirLight(l, WorldPos, Normal, ViewVec, CamDist, 0, iDiffuse, iSpecular);
	}

	[unroll] for(int l=0; l < spotlights; l++)
	{
		DoSpotLight(l, WorldPos, Normal, ViewVec, 0, iDiffuse, iSpecular);
	}

	[unroll] for(int l=0; l < pointlights; l++)
	{
		DoPointLight(l, WorldPos, Normal, ViewVec, 0, iDiffuse, iSpecular);
	}

    //col.rgb *= min((1-shadow), i.Diffuse) + AmbientLight;
    
	float3 Color = (g_AmbientLight + iDiffuse) * ColorMap;

	Color.rgb = Color * i.Extinction + i.Inscatter;

    return float4(Color, ColorMap.a);
    //return float4(-i.AlphaBetaGamma.yyy*0.1, 1);
}

stateblock RenderStates = stateblock_state
{
    ZEnable = true;
    ZWriteEnable = false;
    ZFunc = LESSEQUAL;

	SlopeScaleDepthBias = -0.999;
	DepthBias = -0.00005;
    Lighting = false;
    Cullmode = (g_CullMode);
    AlphaBlendEnable = true;
    AlphaTestEnable = true; 
    AlphaRef = 0;  
    AlphaFunc = GREATER; 
    BlendOp = ADD;
    SrcBlend = SRCALPHA;
    DestBlend = INVSRCALPHA;
};

#ifndef NUMDIR
#define NUMDIR 0
#endif

#ifndef NUMSPOT
#define NUMSPOT 0
#endif

#ifndef NUMPOINT
#define NUMPOINT 0
#endif

technique RenderCurve
{
    pass P0
    {
        StateBlock = (RenderStates);
        
        VertexShader = compile vs_3_0 CurveVS();
        PixelShader  = compile ps_3_0 CurvePS(NUMDIR, NUMSPOT, NUMPOINT);
    }
}


















