
struct STScatterParam
{
	float4 SunColorIntensity;
	float3 BetaRayleigh;
	float3 BetaDashRayleigh;
	float3 BetaMie;
	float3 BetaDashMie;
	float  HG;
	float3 SunLightDir;
}g_SCParam;

struct STInscatter
{
	float3 Inscatter;
	float3 Extinction;
};

static const float FarPlane = 2000;
static const float DimensionScale = 7.5;
static const float3 InscatterBias = {0.01, 0.01, 0.012};
//static const float3 InscatterBias = {0, 0, 0};

STInscatter Inscatter(float3 Point, float3 CamPos)
{
	Point *= DimensionScale;
	CamPos *= DimensionScale;

	//float3 PointDir = (Point - CamPos);
	float3 PointDir = (CamPos - Point);
	float PointDist = length(PointDir);
	PointDir /= PointDist; //normalize point dir

	//float3 DirPoint = (g_SCParam.CamPos - Point) * DimensionScale;
	
	//float PointHeight = Point.z;
	//float PointDist = length(DirPoint);
	//float CosTheta = dot(g_SCParam.SunLightDir, DirPoint/PointDist); 
	float CosTheta = dot(g_SCParam.SunLightDir, PointDir); 

	//determine rayleigh and mie optical depth (approximation)
	//scattering strength a height of point
	float fRayleigh = exp(-max(Point.z, 0) / 15000.0f) / 2;
	float fMie = exp(-max(Point.z, 0) / 5000.0f) / 2;

	//average with scattering at camera height
	//float CamHeight = SC_CamPos.z * DimensionScale;
	fRayleigh += exp(-max(CamPos.z, 0) / 15000.0f) / 2;
	fMie += exp(-max(CamPos.z, 0) / 5000.0f) / 2;

	fRayleigh *= PointDist;
	fMie *= PointDist / 3;

    // calculateextinction terms for inscateredextinction
    float3 extinction = exp(-g_SCParam.BetaRayleigh * fRayleigh - g_SCParam.BetaMie * fMie); //e^(-beta * depth)
    
	float thetaRay = CosTheta * CosTheta / 2 + 2;

    //calculate mie scatering term
    //Phase2 ( theta ) = (1-g^2) /(1+g^2-2g * cos ( theta ) ) ^(3/2)
    float thetaMie = (1 - g_SCParam.HG * g_SCParam.HG) / pow(1 + g_SCParam.HG * g_SCParam.HG - 2 * g_SCParam.HG * CosTheta, 3.f / 2.f );

	// Inscattering ( I ) = ( Beta_R * Phase_R( theta ) + Beta_M * Phase_M( theta ) ) *
    // [1-exp(-Beta_R*s ).exp(-Beta_M*s ) ] / ( Beta_R + Beta_M)
    float3 inscatter = (g_SCParam.BetaDashRayleigh * thetaRay + g_SCParam.BetaDashMie * thetaMie) * (1 - extinction) / (g_SCParam.BetaRayleigh + g_SCParam.BetaMie);

    inscatter = saturate(inscatter);

    inscatter *= g_SCParam.SunColorIntensity.xyz;
    inscatter *= g_SCParam.SunColorIntensity.w; 

	inscatter += InscatterBias;

	STInscatter o;

	o.Inscatter = inscatter;

	//interpolate extinction to exponetial fog
	float rangeext = saturate(PointDist / (FarPlane * DimensionScale));
	float fDistFog = pow(rangeext, 4);

	//o.Extinction = (1-rangeext);
	//o.Extinction = extinction;
	//o.Extinction = saturate(extinction + 1 - rangeext);
	o.Extinction = lerp(extinction, 0, fDistFog);

	return o;
}

float3 InscatterInf(const float3 ViewDir, float3 CamPos)
{
	CamPos *= DimensionScale;

	float CosTheta = dot(g_SCParam.SunLightDir, -ViewDir);
    //float thetaRay = cosTheta * cosTheta + 1;
    float thetaRay = CosTheta * CosTheta / 2 + 2;

	//return saturate(cosTheta);
	//return saturate(thetaRay);

	//real:
    //float distRayleigh = 15000 * exp(-max(CamPos.z, 0) / 15000.0f); //*PointDist (=15000)
    //float distMie = 5000 * exp(-max(CamPos.z, 0) / 5000.0f); //*PointDist / 3 (=5000)

	//to be same with objects and terrain:
	float fRayleigh = exp(0 / 15000.0f) / 2;
	float fMie = exp(0 / 5000.0f) / 2;
	fRayleigh += exp(-max(CamPos.z, 0) / 15000.0f) / 2;
	fMie += exp(-max(CamPos.z, 0) / 5000.0f) / 2;

	fRayleigh *= FarPlane * DimensionScale;
	fMie *= FarPlane * DimensionScale / 3;

	//distRayleigh *= DimensionScale;
	//distMie *= DimensionScale;

    // calculateextinction terms for inscateredextinction
    float3 extinction = exp(-g_SCParam.BetaRayleigh * fRayleigh - g_SCParam.BetaMie * fMie);

	//return float4(extinction, 1);
    
    //calculate mie scatering term
    //Phase2 ( theta ) = (1-g^2) /(1+g^2-2g * cos ( theta ) ) ^(3/2)
    float thetaMie = (1 - g_SCParam.HG * g_SCParam.HG) / pow(1 + g_SCParam.HG * g_SCParam.HG - 2 * g_SCParam.HG * CosTheta, 3.f / 2.f );

	//return saturate(thetaMie);

    // Inscattering ( I ) = ( Beta_R * Phase_R( theta ) + Beta_M * Phase_M( theta ) ) *
    // [1-exp(-Beta_R*s ).exp(-Beta_M*s ) ] / ( Beta_R + Beta_M)
    float3 inscatter = (g_SCParam.BetaDashRayleigh * thetaRay + g_SCParam.BetaDashMie * thetaMie) * (1 - extinction) / (g_SCParam.BetaRayleigh + g_SCParam.BetaMie);

	//return float4(g_SCParam.BetaMie * 10, 1);

    inscatter = saturate(inscatter);

    inscatter *= g_SCParam.SunColorIntensity.xyz;
    inscatter *= g_SCParam.SunColorIntensity.w; 

	//inscatter = 0;
	//return g_SCParam.SunColorIntensity.w;

	return inscatter + InscatterBias;
}