// shadowmapping utilities

float4x4	SM_matSM;
float		SM_fShadowMapSize;

float		SM_DetphBias;
float		SM_SlopeBias;

texture SM_ShadowMap;
sampler SM_sShadowMap = sampler_state
{
    Texture = <SM_ShadowMap>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = POINT;
    MinFilter = POINT;
};

#define SHADOW_EPSILON 0.00003f

//this will be preshadered
float4x4 GetMatSMTex(void)
{
	float off = 0.5f + 0.5f / SM_fShadowMapSize;
	//float off = 0.5f + 0.5f * SM_rcpShadowMapSize;
	return mul(SM_matSM, float4x4(	0.5f, 0.0f, 0.0f, 0.0f,
									0.0f, -0.5f, 0.0f, 0.0f,
									0.0f, 0.0f, 1, 0.0f,
									off, off, 0.0f, 1.0f));
}

float4 CalcSMTexC(const float4 p)
{
	return mul(p, GetMatSMTex());
	//float4 temptexc = mul(p, matSMTex);
	//return temptexc.xy/temptexc.w;
}

float2 CalcDepth(const float4 p)
{
	return (mul(p, SM_matSM)).zw;
}

float CalcShadow_Simple(const float4 ShadowMapTexC, const float Depth)
{
	return ((tex2Dproj(SM_sShadowMap, ShadowMapTexC)).r + SHADOW_EPSILON < Depth)? 1.0f: 0.0f;  
}

float CalcShadow_4xPCF(const float2 ShadowMapTexC, const float Depth)
{
	float2 texelpos = ShadowMapTexC * SM_fShadowMapSize;
    float2 lerps = frac( texelpos );

	const float rcpShadowMapSize = 1 / SM_fShadowMapSize;

	float sourcevals[4];
	sourcevals[0] = (tex2D( SM_sShadowMap, ShadowMapTexC ).r + SHADOW_EPSILON < Depth)? 1.0f: 0.0f;  
    sourcevals[1] = (tex2D( SM_sShadowMap, ShadowMapTexC + float2(rcpShadowMapSize, 0) ).r + SHADOW_EPSILON < Depth)? 1.0f: 0.0f;  
    sourcevals[2] = (tex2D( SM_sShadowMap, ShadowMapTexC + float2(0, rcpShadowMapSize) ).r + SHADOW_EPSILON < Depth)? 1.0f: 0.0f;  
    sourcevals[3] = (tex2D( SM_sShadowMap, ShadowMapTexC + float2(rcpShadowMapSize, rcpShadowMapSize) ).r + SHADOW_EPSILON < Depth)? 1.0f: 0.0f;  
        
    // lerp between the shadow values to calculate the light amount
    return lerp( lerp( sourcevals[0], sourcevals[1], lerps.x ), lerp( sourcevals[2], sourcevals[3], lerps.x ), lerps.y );
}

float4 DepthPS(float2 iDepth : TEXCOORD0) : COLOR
{
	return iDepth.x / iDepth.y;
}