float4x4    g_matProj : PROJ;
float4x4    g_matView : VIEW;
float4x4    g_matVP : VIEWPROJ;
float4x4	g_matInvVP : INVVIEWPROJ;

//world constants
float       g_BlockSize;
float       g_FieldSize;

//block constants
float2      g_BlockOffset;

//instance constants
float2      g_PatchOffset;
float       g_PatchScale;

float4      g_AmbientLight : AMBIENT;
float3      g_LightDir;
float4      g_SunColor;
float       g_LightStrength;

float3      g_CamPos : CAMPOS;

float3      g_FogColor;
float       g_FogDist;
float		g_FogStrength;

#include "Shaders/Lighting.fx"
#include "Shaders/Scattering.fx"

//float2 BlendScale[4];
//float2 BlendOffset[4];

int g_TexFilter : TEXFILTER;
int g_TexAnisotropy : TEXANISO;
int g_CullMode : CULLMODE;

texture g_HeightMap;
sampler g_sHeightMap = sampler_state
{
    Texture = <g_HeightMap>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = POINT;
    MagFilter = POINT;
    MinFilter = POINT;
};

texture g_NormalMap;
sampler g_sNormalMap = sampler_state
{
    Texture = <g_NormalMap>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = POINT;
    MagFilter = POINT;
    MinFilter = POINT;
};

#define MAXLAYERS 6

float2 g_TexScale[MAXLAYERS];
float4 g_BlendMask[MAXLAYERS];

texture g_Texture0, g_Texture1, g_Texture2, g_Texture3, g_Texture4, g_Texture5;
//texture g_Normal0, g_Normal1, g_Normal2, g_Normal3, g_Normal4, g_Normal5;
texture g_Blend0, g_Blend1, g_Blend2, g_Blend3, g_Blend4, g_Blend5;

//sampler arrays are crap!! there is a bug with setting the textures and additional filter states are ignored

sampler g_sTexture0 = sampler_state
{
	Texture = <g_Texture0>;
	AddressU = Wrap;
	AddressV = Wrap;
	MipFilter = LINEAR;
	MagFilter = LINEAR;
	MinFilter = (g_TexFilter);
	MaxAnisotropy = (g_TexAnisotropy);
};

sampler g_sTexture1 = sampler_state
{
	Texture = <g_Texture1>;
	AddressU = Wrap;
	AddressV = Wrap;
	MipFilter = LINEAR;
	MagFilter = LINEAR;
	MinFilter = (g_TexFilter);
	MaxAnisotropy = (g_TexAnisotropy);
};

sampler g_sTexture2 = sampler_state
{
	Texture = <g_Texture2>;
	AddressU = Wrap;
	AddressV = Wrap;
	MipFilter = LINEAR;
	MagFilter = LINEAR;
	MinFilter = (g_TexFilter);
	MaxAnisotropy = (g_TexAnisotropy);
};

sampler g_sTexture3 = sampler_state
{
	Texture = <g_Texture3>;
	AddressU = Wrap;
	AddressV = Wrap;
	MipFilter = LINEAR;
	MagFilter = LINEAR;
	MinFilter = (g_TexFilter);
	MaxAnisotropy = (g_TexAnisotropy);
};

sampler g_sTexture4 = sampler_state
{
	Texture = <g_Texture4>;
	AddressU = Wrap;
	AddressV = Wrap;
	MipFilter = LINEAR;
	MagFilter = LINEAR;
	MinFilter = (g_TexFilter);
	MaxAnisotropy = (g_TexAnisotropy);
};

sampler g_sTexture5 = sampler_state
{
	Texture = <g_Texture5>;
	AddressU = Wrap;
	AddressV = Wrap;
	MipFilter = LINEAR;
	MagFilter = LINEAR;
	MinFilter = (g_TexFilter);
	MaxAnisotropy = (g_TexAnisotropy);
};

sampler g_sBlend0 = sampler_state
{
	Texture = <g_Blend0>; 
	AddressU = Clamp;
	AddressV = Clamp;
	MipFilter = NONE;
	MagFilter = LINEAR;
	MinFilter = (g_TexFilter);
	MaxAnisotropy = (g_TexAnisotropy);
};

sampler g_sBlend1 = sampler_state
{
	Texture = <g_Blend1>; 
	AddressU = Clamp;
	AddressV = Clamp;
	MipFilter = NONE;
	MagFilter = LINEAR;
	MinFilter = (g_TexFilter);
	MaxAnisotropy = (g_TexAnisotropy);
};

sampler g_sBlend2 = sampler_state
{
	Texture = <g_Blend2>; 
	AddressU = Clamp;
	AddressV = Clamp;
	MipFilter = NONE;
	MagFilter = LINEAR;
	MinFilter = (g_TexFilter);
	MaxAnisotropy = (g_TexAnisotropy);
};

sampler g_sBlend3 = sampler_state
{
	Texture = <g_Blend3>; 
	AddressU = Clamp;
	AddressV = Clamp;
	MipFilter = NONE;
	MagFilter = LINEAR;
	MinFilter = (g_TexFilter);
	MaxAnisotropy = (g_TexAnisotropy);
};

sampler g_sBlend4 = sampler_state
{
	Texture = <g_Blend4>; 
	AddressU = Clamp;
	AddressV = Clamp;
	MipFilter = NONE;
	MagFilter = LINEAR;
	MinFilter = (g_TexFilter);
	MaxAnisotropy = (g_TexAnisotropy);
};

struct VS_INPUT
{
    float2 Pos : POSITION;
};

struct VS_OUTPUT
{
    float4 Pos : POSITION;
    //float3 Normal : COLOR0;
    //c vfloat3 Tangent : COLOR1;
    float4 PosT : TEXCOORD0;
    //float4 SMTexC: TEXCOORD1;
    //float2 Depth: TEXCOORD2;
    float2 BlockPos : TEXCOORD1;
    float2 BlendC : TEXCOORD2;
    float3 Extinction : TEXCOORD3; //inscatter
	float3 Inscatter : TEXCOORD4; //inscatter
	float3 Normal : TEXCOORD5;
	//float3 Tangent : TEXCOORD6;
};

VS_OUTPUT VSTerrain(VS_INPUT i)
{
    VS_OUTPUT o;

    float2 p = i.Pos * g_PatchScale + g_PatchOffset;
    float2 hmtc = (p + 1.5) / (g_BlockSize+3);
    float2 nmtc = (p + 0.5) / (g_BlockSize+1);
    o.BlendC = nmtc;
    o.BlockPos = p / g_BlockSize;
    p = p * g_FieldSize + g_BlockOffset;

    float3 wpos = float3(p, tex2Dlod(g_sHeightMap, float4(hmtc, 0, 0)).r);
    o.Pos = mul(float4(wpos, 1), g_matVP);

    o.PosT = o.Pos;

    float2 nxy = tex2Dlod(g_sNormalMap, float4(nmtc, 0, 0)) * 2 - 1;

    o.Normal = float3(nxy, sqrt(1 - nxy.x*nxy.x - nxy.y*nxy.y));
    
    /*if(bShadow)
    {
        float slope = (1-dot(o.Normal, -g_LightDir));
        float bias = slope * SM_SlopeBias + SM_DetphBias;
        float4 swpos = float4(wpos + bias * -g_LightDir, 1);
        o.SMTexC = CalcSMTexC(swpos);
        o.Depth = CalcDepth(swpos);
    }
    else
    {
        o.SMTexC = 0;
        o.Depth = 0;
    }*/

	//o.SMTexC = 0;
    //o.Depth = 0;

    //if(bRefr) //undwater fog
	if(0)
    {
        //calculate distance of hit with z=0
        float3 CamDirN = normalize(g_CamPos - wpos);
        float t = g_CamPos.z / CamDirN.z;
        float3 HitPos = g_CamPos - t * CamDirN;
        float WaterDist = length(HitPos - wpos);
		float Density = saturate(0.5 + WaterDist * 0.5 / g_FogDist);
		o.Inscatter = Density * g_FogColor;
        o.Extinction = 1 - Density;
    }
    else
    {
		STInscatter ins = Inscatter(wpos, g_CamPos);
		//o.Inscatter = ins.Inscatter * g_FogStrength;
		//o.Extinction = lerp(ins.Extinction,1,1-g_FogStrength);
        o.Inscatter = ins.Inscatter;
		o.Extinction = ins.Extinction;

    }

    return o;
}

float4 PSTerrain(VS_OUTPUT i, uniform int Layers, uniform int dirlights, uniform int spotlights, uniform int pointlights) : COLOR
{
	float4 v = mul(i.PosT, g_matInvVP);
	float3 WorldPos = v.xyz/v.w;

	float3 ViewVec = g_CamPos - WorldPos;
      float CamDist = length(ViewVec);
      ViewVec /= CamDist;
	
	float3 iDiffuse = 0;
	float3 iSpecular = 0;

	float3 Normal = normalize(i.Normal);

	//float3 nn = normalize(i.Normal);
	//float3 nt = normalize(float3(1, 0, 0));
	//float3x3 TBNMatrix = float3x3(nt, cross(nt, nn), nn);

	[unroll] for(int l=0; l < dirlights; l++)
	{
		DoDirLight(l, WorldPos, Normal, ViewVec, CamDist, 0, iDiffuse, iSpecular);
	}

	[unroll] for(int l=0; l < spotlights; l++)
	{
		DoSpotLight(l, WorldPos, Normal, ViewVec, 0, iDiffuse, iSpecular);
	}

	[unroll] for(int l=0; l < pointlights; l++)
	{
		DoPointLight(l, WorldPos, Normal, ViewVec, 0, iDiffuse, iSpecular);
	}

    float3 DiffuseColor = tex2D(g_sTexture0, i.BlockPos * g_TexScale[0]).rgb;
            
	//unpacked sampler array code:
	float2 TexC;
	float Blend;
	float3 TexColor;

	if (Layers > 1)
	{
		TexC = i.BlockPos * g_TexScale[1];
		Blend = dot(tex2D(g_sBlend0, i.BlendC), g_BlendMask[1]);
		TexColor = tex2D(g_sTexture1, TexC);
		DiffuseColor.rgb = lerp(DiffuseColor, TexColor, Blend);

		if (Layers > 2)
		{
			TexC = i.BlockPos * g_TexScale[2];
			Blend = dot(tex2D(g_sBlend1, i.BlendC), g_BlendMask[2]);
			TexColor = tex2D(g_sTexture2, TexC);
			DiffuseColor.rgb = lerp(DiffuseColor, TexColor, Blend);

			if (Layers > 3)
			{
				TexC = i.BlockPos * g_TexScale[3];
				Blend = dot(tex2D(g_sBlend2, i.BlendC), g_BlendMask[3]);
				TexColor = tex2D(g_sTexture3, TexC);
				DiffuseColor.rgb = lerp(DiffuseColor, TexColor, Blend);

				if (Layers > 4)
				{
				      TexC = i.BlockPos * g_TexScale[4];
					Blend = dot(tex2D(g_sBlend3, i.BlendC), g_BlendMask[4]);
					TexColor = tex2D(g_sTexture4, TexC);
					DiffuseColor.rgb = lerp(DiffuseColor, TexColor, Blend);

					if (Layers > 5)
					{
						TexC = i.BlockPos * g_TexScale[5];
						Blend = dot(tex2D(g_sBlend4, i.BlendC), g_BlendMask[5]);
						TexColor = tex2D(g_sTexture5, TexC);
						DiffuseColor.rgb = lerp(DiffuseColor, TexColor, Blend);
					}
				}
			}
		}
	}
   
	float3 Color = (g_AmbientLight.rgb + iDiffuse) * DiffuseColor;

    //float3 Normal = i.Normal;
        
    //float s = min(saturate(dot(Normal, -g_LightDir)), 1-shadow);
    
	//Color.rgb = (s * g_SunColor.rgb + g_AmbientLigth) * Color.rgb;

    Color.rgb = Color * i.Extinction + i.Inscatter;

//    Color.rgb = i.Inscatter;
//    Color.rgb = i.Extinction;

        //float2 nxy = tex2D(SNormalMap, i.BlockPos).xy;
//      return nxy.y * 2 - 1;

    float far = g_matProj[2][3] / (1 - g_matProj[2][2]);
    float depth = 1 - mul(WorldPos, g_matView).z / far;

//return depth;
    return float4(Color, depth);
       // return shadow;
}

void VSShadow(VS_INPUT i, out float4 oPos : POSITION, out float2 oDepth : TEXCOORD0)
{
    float2 p = i.Pos * g_PatchScale + g_PatchOffset;
    float2 hmtc = (p + 1.5) / (g_BlockSize + 3);
    p = p * g_FieldSize + g_BlockOffset;

    float3 wpos = float3(p, tex2Dlod(g_sHeightMap, float4(hmtc, 0, 0)).r);
    
	oPos = mul(float4(wpos, 1), g_matVP);

    oDepth.xy = oPos.zw;
}

float4 DepthPS(float2 iDepth : TEXCOORD0) : COLOR
{
	return iDepth.x / iDepth.y;
}

stateblock RSTerrainBase = stateblock_state
{
    CullMode = (g_CullMode);
    ZFunc = LESS;
    ZEnable = true;
    ZWriteEnable = true;
    AlphaBlendEnable = false;
    AlphaTestEnable = false;
};

technique Render_ShadowMap
{
    pass P0
    {      
		CullMode = NONE;
		ZFunc = LESS;
		ZEnable = true;
		ZWriteEnable = true;
		AlphaBlendEnable = false;
		AlphaTestEnable = false;

        VertexShader = compile vs_3_0 VSShadow();
        PixelShader  = compile ps_3_0 DepthPS();
    }
}

#ifndef NUMDIR
#define NUMDIR 0
#endif

#ifndef NUMSPOT
#define NUMSPOT 0
#endif

#ifndef NUMPOINT
#define NUMPOINT 0
#endif

#define RenderTerrain(lcnt)						\
technique Render_L##lcnt													\
{                                                                           \
    pass P0                                                                 \
    {                                                                       \
        StateBlock = (RSTerrainBase);                                       \
                                                                            \
        VertexShader = compile vs_3_0 VSTerrain();						\
        PixelShader  = compile ps_3_0 PSTerrain(lcnt, NUMDIR, NUMSPOT, NUMPOINT);				\
    }                                                                       \
                                                                            \
}                                                                           
/*                                                                            \
technique Render_L##lcnt##_Shaodw											\
{                                                                           \
    pass P0                                                                 \
    {                                                                       \
        StateBlock = (RSTerrainBase);                                       \
                                                                            \
        VertexShader = compile vs_3_0 VSTerrain(true);						\
        PixelShader  = compile ps_3_0 PSTerrain(true, lcnt);				\
    }                                                                       \
} */                                                                          \

//RenderTerrain(1, NUMDIR, NUMSPOT, NUMPOINT);

RenderTerrain(1);
RenderTerrain(2);
RenderTerrain(3);
RenderTerrain(4);
RenderTerrain(5);
RenderTerrain(6);

/*RenderTerrain(1, 1, 0, 0);
RenderTerrain(2, 1, 0, 0);
RenderTerrain(3, 1, 0, 0);
RenderTerrain(4, 1, 0, 0);
RenderTerrain(5, 1, 0, 0);
RenderTerrain(6, 1, 0, 0);*/

//RenderTerrain_Shadow_L(1)
































