float BlockSize;
float Radius;
float Falloff;
float Strength;
float Height;
float2 Center;
float FieldSize;

float4x4 matCurveToWorld;
float4x4 matWorldToCurve;
float RoadWidth;
float RoadA;
float RoadB;
float RoadLength;

#define PHI 0.0001

int HeightMapFilter;

texture HeightMap;
sampler2D SHeightMap = sampler_state
{
    Texture = <HeightMap>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = (HeightMapFilter);
    MinFilter = (HeightMapFilter);
};

float2  g_ColorMapScale;
texture g_texColorMap;
sampler2D g_sColorMap = sampler_state
{
    Texture = <g_texColorMap>;
    AddressU = WRAP;
    AddressV = WRAP;
    MipFilter = LINEAR;
    MagFilter = LINEAR;
    MinFilter = LINEAR;
};

float4 g_BlendMask;
texture g_texBlend;
sampler2D g_sBlend = sampler_state
{
    Texture = <g_texBlend>;
    AddressU = CLAMP;
    AddressV = CLAMP;
    MipFilter = NONE;
    MagFilter = POINT;
    MinFilter = POINT;
};

struct VS_IN
{
    float2 Pos : POSITION;
};

struct VS_OUT
{
    float4 Pos : POSITION;
    float2 WPos : TEXCOORD0;
};

VS_OUT VSGeneral(VS_IN i)
{
    VS_OUT o;

    //float2 p = i.Pos - 0.5f / (BlockSize+1);
    float2 p = i.Pos / (BlockSize+1);
    o.Pos = float4(p * float2(2, -2) - float2(1, -1), 0 , 1);
    o.WPos = i.Pos;

    return o;
}

VS_OUT VSTerrainBlock(VS_IN i)
{
    VS_OUT o;

    //float2 p = i.Pos - 0.5f / (BlockSize+1);
    float2 p = (i.Pos+1) / (BlockSize+3);
    o.Pos = float4(p * float2(2, -2) - float2(1, -1), 0 , 1);
    o.WPos = i.Pos;

    return o;
}

float4 PSStrength(VS_OUT i) : COLOR
{
    const float d = length(i.WPos - Center);
    if(d <= Radius + PHI)
        return Strength;
    else if(d <= Radius + Falloff)
        return (1 - (d - Radius) / Falloff) * Strength;
    else
        return 0;
}

float4 PSInvStrength(VS_OUT i) : COLOR
{
    const float d = length(i.WPos - Center);
    if(d <= Radius + PHI)
        return 1 - Strength;
    else if(d <= Radius + Falloff)
        return 1 - (1 - (d - Radius) / Falloff) * Strength;
    else
        return 1;
}

float4 PSStrengthSmooth(VS_OUT i) : COLOR
{
    float d = length(i.WPos - Center);
    if(d <= Radius + PHI)
        return float4(Height, Height, Height, Strength);
    else if(d <= Radius + Falloff)
        return float4(Height, Height, Height, Strength * (1 - (d - Radius) / Falloff));
    else 
        return 0.f;
}

float4 PSNormal(VS_OUT i) : COLOR
{
    const float2 p = i.WPos + 1.5;

    const float hx0 = tex2D(SHeightMap, (p + float2(-1, 0)) / (BlockSize + 3));
    const float hx1 = tex2D(SHeightMap, (p + float2(1, 0)) / (BlockSize + 3));
    
    const float hy0 = tex2D(SHeightMap, (p + float2(0, -1)) / (BlockSize + 3));
    const float hy1 = tex2D(SHeightMap, (p + float2(0, 1)) / (BlockSize + 3));
    
    //const float2 dxy = float2(hx0 - hx1, hy0 - hy1);
    //const float2 nxy = sin(atan(dxy / FieldSize / 2));

    const float3 vx = float3(FieldSize * 2, 0, hx0 - hx1);
    const float3 vy = float3(0, FieldSize * 2, hy0 - hy1);
    const float2 nxy = normalize(cross(vy, vx)).xy;

//    return 0;
    return float4(nxy * 0.5 + 0.5, 0, 0);

//  return hx0;
//  return float4(dxy, 0,1);

//  return float4(1,0,1,0);
}

float4 PSRoadHeight(VS_OUT i) : COLOR
{
    float2 CurvePos = mul(float4(i.WPos,0, 1) , matWorldToCurve);

    const float Alpha = 3 * CurvePos.y * (RoadA + RoadB);
    const float Beta = -1 -(4 * RoadA + 2 * RoadB) * CurvePos.y;
    const float Gamma = CurvePos.x + CurvePos.y * RoadA;

    const float u = 2 * Gamma / (sqrt(Beta * Beta - 4 * Alpha * Gamma) - Beta);
    clip(float2(u, -u+1));

    float fu = u*(1-u)*(1-u)*RoadA - u*u*(1-u)*RoadB;
    float dfu = 3*u*u*(RoadA + RoadB) - 2*u*(2*RoadA + RoadB) + RoadA;
    
    const float2 CurveCenterWorld = mul(float4(u, fu, 0, 1), matCurveToWorld);
    const float2 HmTexC = (CurveCenterWorld + 0.5) / (BlockSize+1);
    const float CenterHeight = tex2D(SHeightMap, HmTexC);

    //if(abs(HmTexC.y - 1.5/(BlockSize+3)) < 0.0001)
    //  return 0;

    float2 CurveOffset = CurvePos.xy - float2(u,fu);
    float2 CurveNormal = float2(-dfu,1);
    
    float v = dot(CurveNormal,CurveOffset)/sqrt(dfu*dfu+1);

    v = v * RoadLength/RoadWidth + 0.5;

    //0 - 1 is on road
    clip(float2(v,-v+1));

    return CenterHeight;
    //return 0;
}

float4 PSTextureAllBase(VS_OUT i) : COLOR
{
	float2 tc = i.WPos / BlockSize;
	return tex2D(g_sColorMap, tc * g_ColorMapScale);
}

float4 PSTextureAllLayer(VS_OUT i) : COLOR
{
	float2 tc = i.WPos / BlockSize;
	float2 tcblend = (i.WPos + 0.5f) / (BlockSize+1);
	float blend = dot(tex2D(g_sBlend, tcblend), g_BlendMask);
	return float4(tex2D(g_sColorMap, tc * g_ColorMapScale).rgb, blend);
}

technique ChangeHeight
{
    pass P0
    {      
        CullMode = NONE;
        ZEnable = false;
        ZFunc = ALWAYS;
        ZWriteEnable = false;
        AlphaBlendEnable = true;
        AlphaTestEnable = true;
        AlphaRef = 0;
        AlphaFunc = NOTEQUAL;
        Blendop = ADD;
        SrcBlend = ONE;
        DestBlend = ONE;

        VertexShader = compile vs_3_0 VSGeneral();
        PixelShader  = compile ps_3_0 PSStrength();
    }
}

technique SmoothHeight
{
    pass P0
    {      
        CullMode = NONE;
        ZEnable = false;
        ZFunc = ALWAYS;
        ZWriteEnable = false;
        AlphaBlendEnable = true;
        AlphaTestEnable = true;
        AlphaRef = 0;
        AlphaFunc = NOTEQUAL;
        Blendop = ADD;
        SrcBlend = SRCALPHA;
        DestBlend = INVSRCALPHA;

        VertexShader = compile vs_3_0 VSGeneral();
        PixelShader  = compile ps_3_0 PSStrengthSmooth();
    }
}

technique ComputeNormals
{
    pass P0
    {      
        CullMode = NONE;
        ZEnable = false;
        ZFunc = ALWAYS;
        ZWriteEnable = false;
        AlphaBlendEnable = false;
        AlphaTestEnable = false;
        
        VertexShader = compile vs_3_0 VSGeneral();
        PixelShader  = compile ps_3_0 PSNormal();
    }
}

technique DrawTex
{
    pass P0
    {      
        CullMode = NONE;
        ZEnable = false;
        ZFunc = ALWAYS;
        ZWriteEnable = false;
        AlphaBlendEnable = true;
        AlphaTestEnable = true;
        AlphaRef = 0;
        AlphaFunc = GREATER;
        Blendop = MAX;
        SrcBlend = ONE;
        DestBlend = ONE;

        VertexShader = compile vs_3_0 VSGeneral();
        PixelShader  = compile ps_3_0 PSStrength();
    }
}

technique ClearTex
{
    pass P0
    {      
        CullMode = NONE;
        ZEnable = false;
        ZFunc = ALWAYS;
        ZWriteEnable = false;
        AlphaBlendEnable = true;
        AlphaTestEnable = false;
        //AlphaRef = 0xff;
//        AlphaFunc = LESS;
        Blendop = MIN;
        SrcBlend = ONE;
        DestBlend = ONE;

        VertexShader = compile vs_3_0 VSGeneral();
        PixelShader  = compile ps_3_0 PSInvStrength();
    }
}

technique RenderRoadHeight
{
    pass P0
    {
        CullMode = NONE;
        ZEnable = false;
        ZFunc = ALWAYS;
        ZWriteEnable = false;
        AlphaBlendEnable = false;
        AlphaTestEnable = false;
        
        VertexShader = compile vs_3_0 VSTerrainBlock();
        PixelShader  = compile ps_3_0 PSRoadHeight();
    }
}

technique RenderTextureAllBase
{
    pass P0
    {
        CullMode = NONE;
        ZEnable = false;
        AlphaBlendEnable = false;
        AlphaTestEnable = false;
        
        VertexShader = compile vs_3_0 VSGeneral();
        PixelShader  = compile ps_3_0 PSTextureAllBase();
    }
}

technique RenderTextureAllLayer
{
    pass P0
    {
        CullMode = NONE;
        ZEnable = false;

        AlphaBlendEnable = true;
        AlphaTestEnable = true;
		AlphaFunc = GREATER;
		AlphaRef = 0;

		BlendOp = ADD;
		SrcBlend = SRCALPHA;
		DestBlend = INVSRCALPHA;
        
        VertexShader = compile vs_3_0 VSGeneral();
        PixelShader  = compile ps_3_0 PSTextureAllLayer();
    }
}


































