float Luminance;
float Key;
float WhiteLum;
float AdaptiveB;
float GammaCorrection;

texture Scene;
sampler2D sScene = sampler_state
{
    Texture = <Scene>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = POINT;
    MinFilter = POINT;
};

texture ScaledLum;
sampler2D sScaledLum = sampler_state
{
    Texture = <ScaledLum>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = LINEAR;
    MinFilter = LINEAR;
};

texture LocalAdaptation;
sampler2D sLocalAdaptation = sampler_state
{
    Texture = <LocalAdaptation>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = POINT;
    MinFilter = POINT;
};

float g_GlareStrength;
texture g_texGlare;
sampler2D g_sGlare = sampler_state
{
    Texture = <g_texGlare>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = LINEAR;
    MinFilter = LINEAR;
};

float g_StarStrength;
texture g_texStar;
sampler2D g_sStar = sampler_state
{
    Texture = <g_texStar>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = LINEAR;
    MinFilter = LINEAR;
};

texture g_texBright;
sampler2D g_sBright = sampler_state
{
    Texture = <g_texBright>;
    AddressU = Clamp;
    AddressV = Clamp;
    MipFilter = NONE;
    MagFilter = LINEAR;
    MinFilter = LINEAR;
};

float4 ScalePS(float2 tc : TEXCOORD0) : COLOR
{
    float4 vSample = tex2D(sScene, tc);
    
    //Linear Mapping
    vSample.rgb *= Key / (Luminance + 0.00001f);

	vSample.rgb += tex2D(g_sGlare, tc).rgb * g_GlareStrength;
	//vSample.rgb += tex2D(g_sStar, tc).rgb * g_StarStrength;

    return vSample;
}

float4 LogarithmicPS(float2 tc : TEXCOORD0) : COLOR
{
    float4 vSample = tex2D(sScene, tc);
           
    vSample.rgb = Key * log(vSample + 1) / log(Luminance + 1);

	vSample.rgb += tex2D(g_sGlare, tc).rgb * g_GlareStrength;
	//vSample.rgb += tex2D(g_sStar, tc).rgb * g_StarStrength;

    return vSample;
}

float4 AdaptiveLogarithmicPS(float2 tc : TEXCOORD0) : COLOR
{
    float4 vSample = tex2D(sScene, tc);
         
    vSample.rgb = Key / log10(Luminance+1) * log(vSample.rgb + 1) / log(2 + 8 * pow(vSample.rgb / Luminance, log(AdaptiveB) / log(0.5)) );

	vSample.rgb += tex2D(g_sGlare, tc).rgb * g_GlareStrength;
	//vSample.rgb += tex2D(g_sStar, tc).rgb * g_StarStrength;

    return vSample;
}

float4 ReinhardPS(float2 tc : TEXCOORD0) : COLOR
{
    float4 vSample = tex2D(sScene, tc);

    //Reinhard's Photographic Tone Reproduction Operator
    vSample.rgb *= Key / (Luminance + 0.00001f);
    vSample.rgb = vSample.rgb / (vSample.rgb + 1) ;

	vSample.rgb += tex2D(g_sGlare, tc).rgb * g_GlareStrength;
	//vSample.rgb += tex2D(g_sStar, tc).rgb * g_StarStrength;

    return vSample;
}

float4 ReinhardModPS(float2 tc : TEXCOORD0) : COLOR
{
    float4 vSample = tex2D(sScene, tc);
   
    //Reinhard's Photographic Tone Reproduction Operator (modified)
    vSample.rgb *= Key / (Luminance + 0.00001f);
    vSample.rgb = vSample.rgb * (1 + vSample.rgb / (WhiteLum*WhiteLum)) / (0.5f + vSample.rgb); 
  
	vSample.rgb += tex2D(g_sGlare, tc).rgb * g_GlareStrength;
	//vSample.rgb -= tex2D(g_sBright, tc).rgb * g_GlareStrength *0.3;
	//vSample.rgb += tex2D(g_sStar, tc).rgb * g_StarStrength;

    return vSample;
}

float4 ReinhardLocalPS(float2 tc : TEXCOORD0) : COLOR
{
    float4 color = tex2D(sScaledLum, tc);
    float4 local = tex2D(sLocalAdaptation, tc);
    
    color.rgb /= local.rgb + 1; 
    
    //gamma correction (usually required when mapping images taken by a camera)
    //color.rgb = pow(color.rgb , 1 / GammaCorrection);

    return color;
}

stateblock RenderStates = stateblock_state
{
    ZEnable = false;
    Cullmode = None;
    AlphaBlendEnable = false;
    AlphaTestEnable = false; 
    
};

technique Scale
{
    pass P0
    {
        StateBlock = (RenderStates);

        VertexShader = null;
        PixelShader  = compile ps_3_0 ScalePS();
    }
}

technique Logarithmic
{
    pass P0
    {
        StateBlock = (RenderStates);

        VertexShader = null;
        PixelShader  = compile ps_3_0 LogarithmicPS();
    }
}

technique AdaptiveLogarithmic
{
    pass P0
    {
        StateBlock = (RenderStates);

        VertexShader = null;
        PixelShader  = compile ps_3_0 AdaptiveLogarithmicPS();
    }
}

technique Reinhard
{
    pass P0
    {
        StateBlock = (RenderStates);

        VertexShader = null;
        PixelShader  = compile ps_3_0 ReinhardPS();
    }
}

technique ReinhardMod
{
    pass P0
    {
        StateBlock = (RenderStates);

        VertexShader = null;
        PixelShader  = compile ps_3_0 ReinhardModPS();
    }
}

technique ReinhardLocal
{
    pass P0
    {
        StateBlock = (RenderStates);

        VertexShader = null;
        PixelShader  = compile ps_3_0 ReinhardLocalPS();
    }
}



